/*
	$Id: datum.h,v 1.42 2009/06/05 03:13:50 alex Exp $
	datum.h --- edb datum definition
*/

#ifndef	Datum_Inc_Defined
#define	Datum_Inc_Defined

struct EDB_DATUM {
	void		*super;	/* EdbTupleColumn super; */
	struct EDB_DATUM *n;	/* next datum: EdbDatum */
	u_int8_t	av;
	int		english_shared:1;
	int		japanese_shared:1;
	int		pronounce_shared:1;
	u_int8_t	p_read;
	int32_t	date_from;
	int32_t	date_to;
	eid_t		d_eid;
	utf8_t	*d_english;
	utf8_t	*d_japanese;
	utf8_t	*d_pronounce;
	Link		l_child;
};
typedef	struct EDB_DATUM	*EdbDatum;

#define	DatumSuper(datum)		((datum)->super)
#define	DatumAvail(datum)		((datum)->av)
#define	DatumDateFrom(datum)	((datum)->date_from)
#define	DatumDateTo(datum)	((datum)->date_to)
#define	DatumRead(datum)		((datum)->p_read)

#define	DatumEID(datum)		((datum)->d_eid)
#define	DatumEnglish(datum)	((datum)->d_english)
#define	DatumJapanese(datum)	((datum)->d_japanese)
#define	DatumPronounce(datum)	((datum)->d_pronounce)

#define	DatumEIDisValid(datum)		((datum) && EIDisValid(DatumEID(datum)))
#define	DatumEIDisVALID(datum)		DatumEIDisValid((datum))
#define	DatumEnglishIsValid(datum)	((datum) && STRisValid((datum)->d_english))
#define	DatumJapaneseIsValid(datum)	((datum) && STRisValid((datum)->d_japanese))
#define	DatumPronounceIsValid(datum)	((datum) && STRisValid((datum)->d_pronounce))

#define	DatumEnglishIsUsable(datum)		(DatumEnglishIsValid(datum) && !((datum)->d_english[0]=='.' && (datum)->d_english[1]=='\0'))
#define	DatumJapaneseIsUsable(datum)	(DatumJapaneseIsValid(datum) && !((datum)->d_japanese[0]=='.' && (datum)->d_japanese[1]=='\0'))
#define	DatumPronounceIsUsable(datum)	(DatumPronounceIsValid(datum) && !((datum)->d_pronounce[0]=='.' && (datum)->d_pronounce[1]=='\0'))

#define	DatumIsValid(datum)	((datum) && (EIDisValid(((datum)->d_eid)) || STRisValid((datum)->d_english) || STRisValid((datum)->d_japanese) || STRisValid((datum)->d_pronounce)))

#define	EDB_PICTURE_MAXSIZ	(256*1024)	/* in bytes (binary) */

/* These edb_datum_get_* macro/function are not duplicate datum. */
#define	edb_datum_get_EID(ec, datum)			((datum) ? (datum)->d_eid : 0)
#define	edb_datum_get_English(ec, datum)		(((datum) && STRisValid((datum)->d_english)) ? (datum)->d_english : (utf8_t *)"")
#define	edb_datum_get_Japanese(ec, datum)	(((datum) && STRisValid((datum)->d_japanese)) ? (datum)->d_japanese : (utf8_t *)"")
#define	edb_datum_get_Pronounce(ec, datum)	(((datum) && STRisValid((datum)->d_pronounce)) ? (datum)->d_pronounce : (utf8_t *)"")

#define	edb_datum_get_TEXT(ec, datum)		edb_datum_get_English(ec, datum)
#if 0
#define	edb_datum_get_TEXT2(ec, datum)		edb_datum_get_English(ec, datum)
#endif
#define	edb_datum_get_INT(ec, datum)			((datum) ? ATOI((char *)(datum)->d_english) : 0)
#define	edb_datum_get_REAL(ec, datum)		((datum) ? ATOF((char *)(datum)->d_english) : 0.0)
int		edb_datum_set						__P((EdbContext, EdbDatum, eid_t, utf8_t *, utf8_t *, utf8_t *));
#if 0
#define	edb_datum_GET(ec, datum, eid, en, ja, pr)
#define	edb_datum_get_NAME(ec, datum, eid, en, ja, pr)	edb_datum_GET((ec), (datum), (eid), (en), (ja), (pr))
#define	edb_datum_get_SENTENCE(ec, datum, eid, en, ja)	edb_datum_GET((ec), (datum), (eid), (en), (ja), NULL)
#define	edb_datum_PARAGRAPH(ec, datum, eid, en, ja)		edb_datum_GET((ec), (datum), (eid), (en), (ja), NULL)
#endif
#if 0
#define	edb_datum_get_PASSPHRASE(ec, datum)	edb_datum_get_English(ec, datum)
#define	edb_datum_get_EMAIL(ec, datum)		edb_datum_get_English(ec, datum)
#define	edb_datum_get_URL(ec, datum)		edb_datum_get_English(ec, datum)
#define	edb_datum_get_PHONE(ec, datum)		edb_datum_get_English(ec, datum)
#endif
#define	edb_datum_get_DATE(ec, datum)		edb_datum_get_INT(ec, datum)
#define	edb_datum_get_MONETARY(ec, datum)	edb_datum_get_REAL(ec, datum)
#define	edb_datum_get_BOOL(ec, datum)		(((datum) && STRisValid((datum)->d_english) && !strcmp((datum)->d_english, "true")) ? TRUE : FALSE)
#define	edb_datum_get_YEAR(ec, datum)		edb_datum_get_INT(ec, datum)
#define	edb_datum_get_MINUTE(ec, datum)		edb_datum_get_INT(ec, datum)
int		edb_datum_get_DATE2				__P((EdbContext, EdbDatum, int32_t *, int32_t *));
#if 0
#define	edb_datum_get_PICTURE(ec, datum)		edb_datum_get_English(ec, datum)
#define	edb_datum_get_IPv4CIDR(ec, datum)	edb_datum_get_English(ec, datum)
#define	edb_datum_get_IPv4ADDR(ec, datum)	edb_datum_get_English(ec, datum)
#define	edb_datum_get_INETNAME(ec, datum)	edb_datum_get_English(ec, datum)
#define	edb_datum_get_INETFQDN(ec, datum)	edb_datum_get_English(ec, datum)
#endif
/* End of edb_datum_get_* */

#define	EDB_TYPE_TEXT		0
#define	EDB_TYPE_TEXT2		1
#define	EDB_TYPE_EID			2
#define	EDB_TYPE_INT			3
#define	EDB_TYPE_REAL		4
#define	EDB_TYPE_NAME		5
#define	EDB_TYPE_SENTENCE	6
#define	EDB_TYPE_PARAGRAPH	7
#define	EDB_TYPE_PASSPHRASE	8
#define	EDB_TYPE_EMAIL		9
#define	EDB_TYPE_URL		10
#define	EDB_TYPE_PHONE		11
#define	EDB_TYPE_DATE		12
#define	EDB_TYPE_MONETARY	13
#define	EDB_TYPE_BOOL		14
#define	EDB_TYPE_YEAR		15
#define	EDB_TYPE_DATE2		16
#define	EDB_TYPE_PICTURE		17
#define	EDB_TYPE_IPv4CIDR	18
#define	EDB_TYPE_IPv4ADDR	19
#define	EDB_TYPE_INETNAME	20
#define	EDB_TYPE_INETFQDN	21
#define	EDB_TYPE_CERTIFICATE	22
#define	EDB_TYPE_MINUTE		23
#define	EDB_TYPE_EXTID		24

int	edb_text_isEnglish			__P((EdbContext, utf8_t *));
int	edb_text_isJapanese			__P((EdbContext, utf8_t *));
int	edb_text_isPronounce		__P((EdbContext, utf8_t *));

int	edb_text_isEID				__P((EdbContext, utf8_t *));
int	edb_text_isBool				__P((EdbContext, utf8_t *));
int	edb_text_isInteger			__P((EdbContext, utf8_t *));
int	edb_text_isReal				__P((EdbContext, utf8_t *));
int	edb_text_isName			__P((EdbContext, utf8_t *));
int	edb_text_isSentence			__P((EdbContext, utf8_t *));
int	edb_text_isParagraph		__P((EdbContext, utf8_t *));
int	edb_text_isEmail			__P((EdbContext, utf8_t *));
int	edb_text_isURL				__P((EdbContext, utf8_t *));
int	edb_text_isPhone			__P((EdbContext, utf8_t *));
int	edb_text_isDate			__P((EdbContext, utf8_t *));
int	edb_text_isValidDate			__P((EdbContext, utf8_t *));

int	edb_text_isKana			__P((EdbContext, utf8_t *));

int	edb_text_isIPv4CIDR			__P((EdbContext, utf8_t *));
int	edb_text_isIPv4Address		__P((EdbContext, utf8_t *));
int	edb_text_isInetName			__P((EdbContext, utf8_t *));
int	edb_text_isInetFQDN			__P((EdbContext, utf8_t *));
int	edb_text_isCertificate		__P((EdbContext, utf8_t *));

int	check_addr_in_network		__P((utf8_t *, utf8_t *));

struct EDB_CTYPE_INFO {
	char	*name;						/* type name in EDB */
	struct EDB_TYPE_INFO	*info;		/* type info. in EDB */
	char	*v_minimum;					/* mininum value of type */
	char	*v_maximum;					/* maximum value of type */
	char	*v_default;					/* default value of type */
	char	*cryptography;					/* cryptography */
	struct EDB_COLUMN_INFO	*ci;		/* column information */
};
typedef struct EDB_CTYPE_INFO	*EdbCTypeInfo;

#define	EDB_DATUM_CHECK_FUNC_P	__P((EdbContext, EdbCTypeInfo, EdbDatum))
#define	EDB_DATUM_MODIFY_FUNC_P	__P((EdbContext, EdbCTypeInfo, EdbDatum))

struct EDB_TYPE_INFO {
	char	*name;
	int	typeID;
	int	basicTypeID;
	char	*SQL_type;
	int	multilingual;
	int	(*check_datum)	EDB_DATUM_CHECK_FUNC_P;
	int	(*check0)		__P((EdbContext, utf8_t *));
	int	(*modify)		EDB_DATUM_MODIFY_FUNC_P;
	char	*desc;
	char	*longdesc;
	struct EDB_BTEXT explain;
};
typedef struct EDB_TYPE_INFO	*EdbTypeInfo;

#ifdef	EDB_TYPE_DEFINITION

static int	edb_datum_check_TEXT2			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_INT			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_BOOL			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_YEAR			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_MINUTE		EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_EMAIL			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_URL			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_REAL			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_PHONE			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_DATE			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_DATE2			EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_IPV4CIDR		EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_IPV4ADDR		EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_INETNAME		EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_INETFQDN		EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_CERTIFICATE	EDB_DATUM_CHECK_FUNC_P;
static int	edb_datum_check_PASSPHRASE	EDB_DATUM_CHECK_FUNC_P;

static int	edb_datum_modify_TEXT			EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_TEXT2			EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_NAME			EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_SENTENCE		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_INTEGER		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_REAL			EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_DATE			EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_DATE2		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_PICTURE		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_IPV4CIDR		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_IPV4ADDRESS	EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_INETNAME		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_INETFQDN		EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_CERTIFICATE	EDB_DATUM_MODIFY_FUNC_P;
static int	edb_datum_modify_PASSPHRASE	EDB_DATUM_MODIFY_FUNC_P;

#define	DESC_LATIN		"Latin文字とは日本文化圏でいわゆる半角英数記号文字として知られている文字です．\n"
#define	DESC_UNICODE	"和文に関連するUnicode文字とは日本文化圏でいわゆる全角文字として知られている文字のうち，OSに依存する特殊文字を除いた文字集合です．\n"
#define	DESC_CONVERSION	"全角文字を入力しても，データベース内での標準形に変換する際にLatin文字に変換されることがあります．\n"
#define	DESC_ESCAPE		"上付き文字，下付き文字はそれぞれ\\\\^{superscript}, \\\\_{subscript}のように記入してください．「A\\\\^{x}」は「A\\^{x}」，「B\\\\_{y}」は「B\\_{y}」のようになります．\n"

struct EDB_TYPE_INFO	edb_datum_type_table[] = {{
	"TEXT",
	EDB_TYPE_TEXT,
	EDB_TYPE_TEXT,
	"text",
	FALSE,
	NULL,
	edb_text_isEnglish,
	edb_datum_modify_TEXT,
	"テキスト",
	"普通のテキスト（Latin文字）",
	{
	(utf8_t *)"Type for storing text (only Latin character string).",
	(utf8_t *)"Latin文字のみからなるテキストを記憶するための型です．\n"
	DESC_LATIN
	}
},{
	"TEXT2",
	EDB_TYPE_TEXT2,
	EDB_TYPE_TEXT,
	"text",
	FALSE,
	edb_datum_check_TEXT2,
	edb_text_isEnglish,
	edb_datum_modify_TEXT2,
	"テキスト2語",
	"空白で区切られた2語のテキスト（Latin文字）",
	{
	(utf8_t *)"Type for storing text (only Latin character string).",
	(utf8_t *)"Latin文字のみからなるテキストを記憶するための型です．\n"
	DESC_LATIN
	}
},{
	"EID",
	EDB_TYPE_EID,
	EDB_TYPE_EID,
	"mtext",
	TRUE,
	NULL,
	edb_text_isEID,
	edb_datum_modify_TEXT,
	"参照",
	"情報識別子EID",
	{
	(utf8_t *)"Type for referencing to other information.",
	(utf8_t *)"他の情報を参照するための型です．\n"
	"情報識別子（各情報に固有に割り振られた整数値）を記憶します．"
	}
},{
	"BOOL",
	EDB_TYPE_BOOL,
	EDB_TYPE_BOOL,
	"int4",
	FALSE,
	edb_datum_check_BOOL,
	edb_text_isBool,
	edb_datum_modify_TEXT,
	"論理値",
	"論理値（1:真，0:偽）",
	{
	(utf8_t *)"Type for storing the boolean value.",
	(utf8_t *)"論理値（真，偽）を記憶するための型です．"
	}
},{
	"INTEGER",
	EDB_TYPE_INT,
	EDB_TYPE_INT,
	"int4",
	FALSE,
	edb_datum_check_INT,
	edb_text_isInteger,
	edb_datum_modify_INTEGER,
	"整数",
	"整数（範囲: -20億〜20億）",
	{
	(utf8_t *)"Type for storing the integer value.",
	(utf8_t *)"整数値（範囲: -2**31〜0〜2**31-1）を記憶するための型です．"
	}
},{
	"YEAR",
	EDB_TYPE_YEAR,
	EDB_TYPE_INT,
	"int4",
	FALSE,
	edb_datum_check_YEAR,
	edb_text_isInteger,
	edb_datum_modify_INTEGER,
	"西暦年",
	"西暦年（範囲: 西暦1〜9999年）",
	{
	(utf8_t *)"Type for storing the year (A.D.) value.",
	(utf8_t *)"西暦年（範囲: 西暦1〜9999年）を記憶するための型です．"
	}
},{
	"REAL",
	EDB_TYPE_REAL,
	EDB_TYPE_REAL,
	"float8",
	FALSE,
	edb_datum_check_REAL,
	edb_text_isReal,
	edb_datum_modify_REAL,
	"実数",
	"浮動小数点実数（有効桁数: 16桁）",
	{
	(utf8_t *)"Type for storing the real value.",
	(utf8_t *)"実数値（浮動小数点実数，有効桁数: 16桁）を記憶するための型です．"
	}
},{
	"MONETARY",
	EDB_TYPE_MONETARY,
	EDB_TYPE_REAL,
	"float8",
	FALSE,
	edb_datum_check_REAL,
	edb_text_isReal,
	edb_datum_modify_REAL,
	"金額",
	"金額（千円）",
	{
	(utf8_t *)"Type for storing the monetary value. Unit is thousand of Japanese Yen.",
	(utf8_t *)"金額（単位: 千円）を記憶するための型です．"
	}
},{
	"NAME",
	EDB_TYPE_NAME,
	EDB_TYPE_NAME,
	"mtext",
	TRUE,
	NULL,
	edb_text_isName,
	edb_datum_modify_NAME,
	"名称",
	"英名，和名，和名読みのセット",
	{
	(utf8_t *)"Type for storing the name.",
	(utf8_t *)"名前を記憶するための型です．\n"
	"英文テキストと和文テキスト，和文テキストの読みから成ります．\n"
	"英文テキストにはLatin文字のみ，和文テキストにはLatin文字と和文に関連するUnicode文字，読みにはUnicode文字の平仮名，片仮名と「ー」（長音），空白文字を記述することができます．\n"
	DESC_LATIN
	DESC_UNICODE
	DESC_CONVERSION
	"読みは主にデータベース検索結果出力時のソート（並べ換え）に利用されます．\n"
	"読みには漢字や仮名文字，英単語の部分の読みを書き，記号，数字などに対応する部分は空白をいれておくと，通常の索引に近い順に並べられます．\n"
	DESC_ESCAPE
	"記述内容が英文のみの場合には，英文テキストのみを記入し，和文テキストを記入する必要はありません．\n"
	"英文テキストのみ記入され，和文テキストが省略された場合には，和文参照時に英文テキストの内容が参照されます．\n"
	"特別な場合として他の情報を参照するための情報参照子（データ型: EID）を記憶する場合があります．"
	}
},{
	"SENTENCE",
	EDB_TYPE_SENTENCE,
	EDB_TYPE_SENTENCE,
	"mtext",
	TRUE,
	NULL,
	edb_text_isSentence,
	edb_datum_modify_SENTENCE,
	"文",
	"英文，和文のセット",
	{
	(utf8_t *)"Type for storing the sentence.",
	(utf8_t *)"文を記憶するための型です．\n"
	"英文テキストと和文テキストから成ります．\n"
	"英文テキストにはLatin文字のみ，和文テキストにはLatin文字と和文に関連するUnicode文字を記述することができます．\n"
	DESC_LATIN
	DESC_UNICODE
	DESC_CONVERSION
	DESC_ESCAPE
	"記述内容が英文のみの場合には，英文テキストのみを記入し，和文テキストを記入する必要はありません．\n"
	"英文テキストのみ記入され，和文テキストが省略された場合には，和文参照時に英文テキストの内容が参照されます．\n"
	}
},{
	"PARAGRAPH",
	EDB_TYPE_PARAGRAPH,
	EDB_TYPE_SENTENCE,
	"mtext",
	TRUE,
	NULL,
	edb_text_isParagraph,
	edb_datum_modify_SENTENCE,
	"段落",
	"英文，和文のセット",
	{
	(utf8_t *)"Type for storing the paragraph.",
	(utf8_t *)"段落を記憶するための型です．\n"
	"英文テキストと和文テキストから成ります．\n"
	"英文テキストにはLatin文字のみ，和文テキストにはLatin文字と和文に関連するUnicodeコード文字を記述することができます．\n"
	DESC_LATIN
	DESC_UNICODE
	DESC_CONVERSION
	DESC_ESCAPE
	"記述内容が英文のみの場合には，英文テキストのみを記入し，和文テキストを記入する必要はありません．\n"
	"英文テキストのみ記入され，和文テキストが省略された場合には，和文参照時に英文テキストの内容が参照されます．\n"
	}
},{
	"PASSPHRASE",
	EDB_TYPE_PASSPHRASE,
	EDB_TYPE_PASSPHRASE,
	"text",
	FALSE,
	edb_datum_check_PASSPHRASE,
	edb_text_isEnglish,
	edb_datum_modify_PASSPHRASE,
	"パスフレーズ",
	"認証のためのパスフレーズ",
	{
	(utf8_t *)"Type for storing the secret text, ex. passphrase.",
	(utf8_t *)"パスフレーズなどの秘密性を保持したテキストを記憶するための型です．"
	}
},{
	"EMAIL",
	EDB_TYPE_EMAIL,
	EDB_TYPE_EMAIL,
	"text",
	FALSE,
	edb_datum_check_EMAIL,
	edb_text_isEmail,
	edb_datum_modify_TEXT,
	"メールアドレス",
	"電子メールのアドレス",
	{
	(utf8_t *)"Type for storing the E-mail address.",
	(utf8_t *)"電子メールのアドレスを記憶するための型です．"
	}
},{
	"URL",
	EDB_TYPE_URL,
	EDB_TYPE_URL,
	"text",
	FALSE,
	edb_datum_check_URL,
	edb_text_isURL,
	edb_datum_modify_TEXT,
	"URL",
	"World Wide Webにおいて利用されるUniform Resource Locator",
	{
	(utf8_t *)"Type for storing the Uniform Resource Locator (URL).",
	(utf8_t *)"World Wide Web における Uniform Resource Locator (URL) を記憶するための型です．\n"
	"「http://サーバ名/パス名」のように全て記述してください．"
	}
},{
	"PHONE",
	EDB_TYPE_PHONE,
	EDB_TYPE_PHONE,
	"text",
	FALSE,
	edb_datum_check_PHONE,
	edb_text_isPhone,
	edb_datum_modify_TEXT,
	"電話番号",
	"電話番号",
	{
	(utf8_t *)"Type for storing the telephone number",
	(utf8_t *)"電話番号を記憶するための型です．\n"
	"公衆回線の場合には「市外局番-市内局番-番号」のように全て記入してください．\n"
	"内線の場合には単に「番号」のように記入してください．\n"
	"英文情報の作成時には自動的に「+81-88-656-????」のように国番号を挿入します．"
	}
},{
	"DATE",
	EDB_TYPE_DATE,
	EDB_TYPE_DATE,
	"int4",
	FALSE,
	edb_datum_check_DATE,
	edb_text_isDate,
	edb_datum_modify_DATE,
	"日付",
	"日付（西暦年/月/日）",
	{
	(utf8_t *)"Type for storing the date.",
	(utf8_t *)"日付（西暦年/月/日）を記憶するための型です．\n"
	"データベースでのデータ処理時に必要に応じて和歴に変換します．"
	}
},{
	"DATE2",
	EDB_TYPE_DATE2,
	EDB_TYPE_DATE2,
	"int8",
	FALSE,
	edb_datum_check_DATE2,
	edb_text_isDate,
	edb_datum_modify_DATE2,
	"期間",
	"期間（西暦年/月/日〜西暦年/月/日）",
	{
	(utf8_t *)"Type for storing the period.",
	(utf8_t *)"期間（西暦年/月/日〜西暦年/月/日）を記憶するための型です．最終日が未定の場合などには，西暦9999年99月99日を指定してください．\n"
	"データベースでのデータ処理時に必要に応じて和歴に変換します．"
	}
},{
	"PICTURE",
	EDB_TYPE_PICTURE,
	EDB_TYPE_PICTURE,
	"text",
	FALSE,
	NULL,
	edb_text_isEnglish,
	edb_datum_modify_PICTURE,
	"画像",
	"イメージ",
	{
	(utf8_t *)"Type for storing the picture",
	(utf8_t *)"画像を記憶するための型です．"
	}
},{
	"IPv4CIDR",
	EDB_TYPE_IPv4CIDR,
	EDB_TYPE_IPv4CIDR,
	"text",
	FALSE,
	edb_datum_check_IPV4CIDR,
	edb_text_isIPv4CIDR,
	edb_datum_modify_IPV4CIDR,
	"ネットワークアドレス",
	"ネットワークアドレス(IPv4)",
	{
	(utf8_t *)"Type for storing the IPv4 network address (CIDR)",
	(utf8_t *)"ネットワークアドレス(IPv4)を記憶するための型です．"
	}
},{
	"IPv4ADDR",
	EDB_TYPE_IPv4ADDR,
	EDB_TYPE_IPv4ADDR,
	"text",
	FALSE,
	edb_datum_check_IPV4ADDR,
	edb_text_isIPv4Address,
	edb_datum_modify_IPV4ADDRESS,
	"IPアドレス",
	"IPアドレス(IPv4)",
	{
	(utf8_t *)"Type for storing the IPv4 address",
	(utf8_t *)"IPv4アドレスを記憶するための型です．"
	}
},{
	"INETNAME",
	EDB_TYPE_INETNAME,
	EDB_TYPE_INETNAME,
	"text",
	FALSE,
	edb_datum_check_INETNAME,
	edb_text_isInetName,
	edb_datum_modify_INETNAME,
	"InetName",
	"InetName",
	{
	(utf8_t *)"Type for storing the DNS name",
	(utf8_t *)"DNS名(1層)を記憶するための型です．"
	}
},{
	"INETFQDN",
	EDB_TYPE_INETFQDN,
	EDB_TYPE_INETFQDN,
	"text",
	FALSE,
	edb_datum_check_INETFQDN,
	edb_text_isInetFQDN,
	edb_datum_modify_INETFQDN,
	"FQDN",
	"FQDN",
	{
	(utf8_t *)"Type for storing the FQDN",
	(utf8_t *)"FQDNを記憶するための型です．"
	}
},{
	"CERTIFICATE",
	EDB_TYPE_CERTIFICATE,
	EDB_TYPE_CERTIFICATE,
	"text",
	FALSE,
	edb_datum_check_CERTIFICATE,
	edb_text_isCertificate,
	edb_datum_modify_CERTIFICATE,
	"証明書",
	"X.509証明書(PEM形式)",
	{
	(utf8_t *)"Type(paste) for storing the X.509 certificate of server or user.",
	(utf8_t *)"サーバ証明書やユーザ証明書などのX.509証明書を記憶するための型です．"
	}
},{
	"MINUTE",
	EDB_TYPE_MINUTE,
	EDB_TYPE_INT,
	"int4",
	FALSE,
	edb_datum_check_MINUTE,
	edb_text_isInteger,
	edb_datum_modify_INTEGER,
	"分",
	"分（範囲: 0〜）",
	{
	(utf8_t *)"Type for storing the minute value.",
	(utf8_t *)"時間を分単位で記憶するための型です．"
	}
},{
	"EXTID",
	EDB_TYPE_EXTID,
	EDB_TYPE_TEXT,
	"text",
	FALSE,
	NULL,
	edb_text_isEnglish,
	edb_datum_modify_TEXT,
	"外部DB識別子",
	"外部DBの識別子",
	{
	(utf8_t *)"Type for storing external database identifier (only Latin character string).",
	(utf8_t *)"外部DBの識別子を記憶するための型です．\n"
	DESC_LATIN
	}
}};

int		edb_datum_type_num = Ns(edb_datum_type_table);

#else	/* EDB_TYPE_DEFINITION */

extern	struct EDB_TYPE_INFO	edb_datum_type_table[];
extern	int edb_datum_type_num;

#endif	/* EDB_TYPE_DEFINITION */

EdbTypeInfo	edb_datum_type_to __P((EdbContext, char *));

#define		edb_datum_type_to_sql_type(c, s)	(edb_datum_type_to((c), (s))->SQL_type)

EdbTypeInfo	edb_datum_typeID_to __P((EdbContext, int));

#define	DATE_ROUND(d)	(((d)<EDB_DATE_MIN) ? EDB_DATE_MIN : ((EDB_DATE_MAX<(d)) ? EDB_DATE_MAX : (d)))
#define	DATE_Y(d)	(((d)/10000)%10000)
#define	DATE_FiscalY(d)	((((d)-400)/10000)%10000)
#define	DATE_M(d)	(((d)/100)%100)
#define	DATE_D(d)	((d)%100)
#define	YMD_DATE(y, m, d)	(((y)%10000)*10000 + ((m)%100)*100 + (d)%100)

#define	TIME_H(t)	(((t)/10000)%100)
#define	TIME_M(t)	(((t)/100)%100)
#define	TIME_S(t)	((t)%100)
#define	HMS_TIME(h, m, s)	(((h)%100)*10000 + ((m)%100)*100 + (s)%100)

/* datum */
EdbDatum	edb_datum_new				__P((EdbContext, void *));
void		edb_datum_free				__P((EdbContext, EdbDatum));
EdbDatum	edb_datum_dup					__P((EdbContext, void *, EdbDatum));
int		edb_datum_diff					__P((EdbContext, EdbDatum, EdbDatum));
int		edb_datum_check				__P((EdbContext, EdbCTypeInfo, EdbDatum));
EdbDatum	edb_datum_parse_xml_node		__P((EdbContext, void *, EdbCTypeInfo, xmlDocPtr, xmlNodePtr));
utf8_t	*	edb_datum_make_pgsql_value		__P((EdbContext, EdbCTypeInfo, EdbDatum));
utf8_t	*	edb_datum_make_pgsql_variant_value	__P((EdbContext, EdbCTypeInfo, EdbDatum));
utf8_t	*	edb_datum_make_xml			__P((EdbContext, EdbCTypeInfo, EdbDatum, utf8_t **, utf8_t **, int));

int		edb_datum_is_empty				__P((EdbContext, EdbDatum));
int		edb_datum_at_present			__P((EdbContext, EdbDatum));
int		edb_datum_at_future				__P((EdbContext, EdbDatum));

void		edb_datum_linkto_type_description	__P((EdbContext, EdbTypeInfo, Doc));
Link		edb_datum_get_reference_list		__P((EdbContext, EdbDatum));
int		edb_datum_check_text			__P((EdbContext, EdbDatum));
int		edb_datum_check_reference		__P((EdbContext, EdbDatum, eid_t));

int		edb_datum_period_overlapped		__P((EdbContext, EdbDatum, EdbDatum));
int		edb_datum_period_compare_for_sort_ascend	__P((LinkObj, LinkObj));
int		edb_datum_period_compare_for_sort_descend	__P((LinkObj, LinkObj));

EdbCTypeInfo	edb_datum_ctype_new		__P((EdbContext, struct EDB_COLUMN_INFO *, char *));
void			edb_datum_ctype_free		__P((EdbContext, EdbCTypeInfo));

#endif	/* Datum_Inc_Defined */

