/*
	$Id: edb.h,v 1.89 2011/04/18 09:45:28 alex Exp $
	edb.h --- header file for EDB
*/

#ifndef	EDB_Inc_Defined
#define	EDB_Inc_Defined

#ifndef	TRUE
#define	TRUE	(1)
#endif

#ifndef	FALSE
#define	FALSE	(0)
#endif

#ifndef	ERROR
#define	ERROR	(-1)
#endif

#include	<stdio.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<sys/types.h>
#include	<sys/time.h>
#include	<time.h>
#include	<string.h>
#include	<pthread.h>
#include	"edb/site.h"
#include	"edb/link.h"

/* システム */
double	system_loadaverage		__P((void));

#define	Ns(x)	((int)(sizeof(x)/sizeof((x)[0])))

#include	"edb/unicode.h"

#define	STRDUP(s)		(strcpy((char *)alloca(sizeof(char)*(strlen(s)+1)), (s)))
Link	lstr_append_printf		__P((Link, char *, ...));
#define	FREE(s)			if((s)!=NULL) { free((s)); (s)=NULL; }
#define	STRALLOC(n)		((char *)alloca(sizeof(char)*(n)))

#include	"edb/bitree.h"

/* 文字列操作関数 */
#define	STRisVALID(s)			((s)!=NULL && (s)[0]!='\0')
#define	STRisValid(s)			((s)!=NULL && (s)[0]!='\0')
#define	ATOI(s)				((s)!=NULL ? atoi((s)) : 0)
#define	ATOF(s)				((s)!=NULL ? atof((s)) : 0.0)
#define	ATOL(s)				((s)!=NULL ? atol((s)) : 0L)
#define	ATOQ(s)				((s)!=NULL ? strtoq((s), NULL, 10) : (quad_t)0)
char	* str_alloc					__P((int));
void	str_capitalize				__P((char *, char *));
void	str_minuscule				__P((char *, char *));
void	str_capitalize1				__P((char *));
void	str_minuscule1				__P((char *));
int	str_isascii					__P((char *));
int	str_isdigit					__P((char *));
int	str_iskana					__P((char *));
#if 0
int	wstr_isfp					__P((wchar16_t *));
wchar16_t	* wstr_skip_whitespace	__P((wchar16_t *));
wchar16_t	* wstr_skip_until_whitespace	__P((wchar16_t *));
wchar16_t	* wstr_concatenate_wstr	__P((Link, wchar16_t *));
Link	wstr_split_bytoken			__P((wchar16_t *));
Link	wstr_duplicate				__P((Link));
Link	wstr_duplicate_only_ascii		__P((Link));
Link	wstr_duplicate_only_katakana	__P((Link));
Link	wstr_uniq					__P((Link));
Link	wstr_caseuniq				__P((Link));
Link	wstr_casesort_and_caseuniq	__P((Link));
Link	wstr_sort_and_uniq			__P((Link));
void	wstr_toascii				__P((wchar16_t *));
wchar16_t	wstr_end_char			__P((wchar16_t *));
#endif
char	* str_skip_whitespace		__P((char *));
char	* str_skip_until_whitespace	__P((char *));
void	str_convert_to_standard		__P((char *));
void	str_standard				__P((char *));
void	str_oneline				__P((char *));
void	str_onephrase				__P((char *));
Link	str_split					__P((char *, int));
char	* str_concatenate			__P((Link, char));
char	* str_concatenate0			__P((Link));
char	* str_concatenate_str		__P((Link, char *));
Link	str_duplicate				__P((Link));
int	str_match					__P((Link, char *));
int	str_casematch				__P((Link, char *));
void	str_sort					__P((Link));
void	wstr_sort					__P((Link));
void	wstr_rsort				__P((Link));
void	wstr_casesort				__P((Link));
Link	str_uniq					__P((Link));
int	str_whats_kind_of_code		__P((unsigned char *));
char *	str_get_phrase			__P((char **));

#if 0
char	* gettodaystr();
char	* getdatestr();
char	* getdatestr_before_day();
char	* getdatestr_before_week();
char	* getdatestr_before_month();
char	* getdatestr_before_year();
#endif

#define	max(x,y)	((x)>(y)?(x):(y))
#define	min(x,y)	((x)<(y)?(x):(y))

#include	"edb/eid.h"

#include	"edb/doc.h"
#include	"edb/catalogue.h"


struct _EDB_DBPATH {
	char	*path;
	char	*host;
	char	*port;
	char	*dbname;
	int	replica;
};
typedef	struct _EDB_DBPATH	*EdbDBPath;

struct _EDB_PG_CONN {
	void		*conn;
	int		transaction;
	eoid_t	age;
	pthread_t	thread;
#define	EDB_PROCESS_IDLE		0
#define	EDB_PROCESS_EXECUTING	1
#define	EDB_PROCESS_TERMINATED	2
	int		state;
	int		number_of_executed;
	int		number_of_switched;
	/* command queue will be sent to backend (only use nonblocking mode) */
	Link			l_pgsql_command_queue;
	utf8_t		*last_command;
};
typedef	struct _EDB_PG_CONN	*EdbPGConn;

struct EDB_CONTEXT {
	EdbPGConn	pg_connections;
	int			number_of_pg_connections;

	int			process_initialized;
	int			process_last_created;
	int			process_number_of_switched;
	pthread_attr_t	process_attr;
	pthread_mutexattr_t	process_mutexattr;
	pthread_mutex_t	process_mutex;
	pthread_cond_t	process_cond;

	EdbPGConn	pg_conn;		// current active connection
	Link			l_dbpath;
	char			*host;
	char			*port;
	char			*dbName;
	eoid_t		age;
	int			number_of_replica;
	char			*options;
	char			*tty;
	char			*pgsqlUser;
	char			*pgsqlPassword;
	FILE			*trace;
	int			pg_server_version;	/* 0x000M0m0p */
#define	PG_SV_mmp(ma, mi, pa)	(((ma)<<16)|((mi)<<8)|(pa))
#define	PG_SV_cmp(ec, ma, mi, pa)	(((ec)->pg_server_version)-PG_SV_mmp((ma), (mi), (pa)))
#define	PG_SV_major(ec)	((((ec)->pg_server_version)>>16)&0xff)
#define	PG_SV_minor(ec)	((((ec)->pg_server_version)>>8)&0xff)
#define	PG_SV_patch(ec)	((((ec)->pg_server_version))&0xff)
	int			transaction;
	int			locked;
	int			japanese;
	int			masterUID;
	int			master;
	int			staff;
	int			Odin;
	char			*remote_addr;
	Link			l_mynetwork;
	int			university;
	time_t		now;			/* UNIX time */
	int			present_date;		/* YYYYMMDD (8 digits) */
	int			present_wday;		/* day of week (Sunday = 0) */
	int			present_time;		/* HHMMSS (6 digits) */
	int			readonly;

	int			chronological_order;

#define	MAX_MAPTO_RECURSE	10
	int			max_mapto_recurse;

#define	MAX_HIERARCHY_RECURSE	10
	int			max_hierarchy_recurse;

	int			pgsql_revolver_cmds;

	int			number_of_query;
	int			number_of_command;
	int			number_of_list_hit;

#define	EDB_BROWSER_URL_DEFAULT		"/cgi-bin/edb_browse"
	char			*browser_url;

	EdbCatalogue	ca_all_tables;
	EdbCatalogue	ca_tables;

	/* eid */
	BiTree		bt_eid_list;
	BiTree		bt_eoid_list;
	eid_t			operator;	/* current operator */

	/* caption */
	BiTree		bt_caption_omit;
	Link			l_caption_omit;

	/* user data */
	void			* user_data;

	/* statistics */
	int			open_trials;
	/* timeval */
	int			trace_time;
	struct timeval	configured_time;

	EdbCatalogue	ca_table_name2eid;

	/* garbage */
	Link	l_garbage;		/* to be free by link_free() */
	Link	l_garbage2;	/* to be free by link_free2() */
	EdbCatalogue	ca_debugger;

	int	edb_datum_text_sharable;	/* sharable text string */

	int	delayed_free;
	Link	l_delayed_link_free2;
	void	*delayed_free_tuple;

	int			verbose;

	EdbCatalogue	ca_auth_users;
	int			dont_use_reldb;
};
typedef	struct EDB_CONTEXT	*EdbContext;

extern	int	_edb_delayed_free_mode;
extern	void	*_edb_delayed_free_bitree;
extern	void	*_edb_delayed_free_link_atom;
extern	void	*_edb_delayed_free_tuple_column;
extern	void	*_edb_delayed_free_datum;
extern	void	*_edb_delayed_free_index;
extern	void	*_edb_delayed_free_xmldb;
extern	int	edb_delayed_free_objects_count;
extern	int	edb_delayed_free_executed_count;
extern	int	edb_delayed_free_recycle_count;

#define	EdbDelayedFreePush(list, obj)	{	*(void **)(obj) = (list);	(list) = (obj);	}
#define	EdbDelayedFreePop(list, obj)	{	(obj) = (list);	(list) = *(void **)(obj);	}

#define	EdbLinkFree(link)		if((link)) { link_free((link)); (link) = NULL; }

#define	EdbLinkFree2(ec, link)	\
	if((link)) {	\
		if((ec)->delayed_free) {	\
			(ec)->l_delayed_link_free2 = link_append((link), (ec)->l_delayed_link_free2);	\
		} else {	\
			link_free2((ec), (link));	\
		}	\
		(link) = NULL;	\
	}


double	edb_loadaverage			__P((EdbContext));

#include	"edb/xml.h"
#include	"edb/db.h"

#define	EDB_DATE_MIN	XML_AN_DATE_FROM_DEFAULT
#define	EDB_DATE_MAX	XML_AN_DATE_TO_DEFAULT

#define	EDB_EID_SEQUENCE_NAME	"edb_eid_seq"
#define	EDB_EOID_SEQUENCE_NAME	"edb_eoid_seq"
#define	EDB_EID_START	10001
#define	EIDisValid(eid)		((eid)>=10001)
#define	EIDisVALID(eid)	EIDisValid((eid))
#define	EOIDisValid(eoid)	((eoid)>=10001)
#define	EOIDisVALID(eoid)	EOIDisValid((eoid))

#include	"edb/xmldb.h"

#define	EDB_INDEX_TABLE				"edb_index"

#define	EDB_COLUMN_FMT		"c%d"

#define	EDB_BUFSIZ	(1024*128)

struct EDB_MTEXT {
	utf8_t	*english;
	utf8_t	*japanese;
	utf8_t	*pronounce;
};
typedef	struct EDB_MTEXT	*EdbMtext;

struct EDB_BTEXT {
	utf8_t	*english;
	utf8_t	*japanese;
};
typedef	struct EDB_MTEXT	*EdbBtext;

#include	"edb/base.h"
#include	"edb/index.h"

#include	"edb/datum.h"

struct EDB_PERMISSION {
	int	p_read;
	int	p_create;
	int	p_write;
	int	p_delete;
};

#include	"edb/table.h"
#include	"edb/tuple.h"
#include	"edb/tcdatum.h"
#include	"edb/person.h"
#include	"edb/personification.h"
#include	"edb/organization.h"
#include	"edb/picture.h"
#include	"edb/condition.h"

#define	EDB_LATEST_VIEW_EXTENSION	"_latest"

/* wellknown table, column, tuple */
#include	"edb/wellknown.h"

#include	"edb/caption.h"
#include	"edb/auth.h"

EdbContext	edb_configure	__P((char *));
void	edb_dbpath_reset			__P((EdbContext));
void	edb_dbpath_set			__P((EdbContext, char *, int));
void	edb_configure_propagate_authority	__P((EdbContext, EdbContext));
void	edb_configure_master_authority	__P((EdbContext));
void	edb_configure_user_authority	__P((EdbContext, eid_t));
#define	IsEdbDebugger(ec, eid)	(((ec)->ca_debugger!=NULL) && edb_catalogue_lookup((ec)->ca_debugger, (eid))!=NULL)

/* session */
eid_t		edb_session_operator			__P((EdbContext));
eid_t		edb_session_effective_operator		__P((EdbContext));
int		edb_session_delete				__P((EdbContext, utf8_t *));
int		edb_session_store				__P((EdbContext, utf8_t *));
int		edb_session_number_of_users		__P((EdbContext));
int		edb_session_touch				__P((EdbContext, utf8_t *));
int		edb_session_revive				__P((EdbContext, utf8_t *));
int		edb_session_delete_expired		__P((EdbContext));
int		edb_session_auth				__P((EdbContext, eid_t, utf8_t *));
int		edb_session_auth_staff			__P((EdbContext, eid_t, utf8_t *));
int		edb_session_auth_Odin			__P((EdbContext, eid_t, utf8_t *));
int		edb_session_session_is_active		__P((EdbContext, utf8_t *));

/* edb */
FILE *	edb_trace_set		__P((EdbContext, FILE *));
int		edb_trace			__P((EdbContext, char *, ...));
int		edb_trace_flush		__P((EdbContext));
int		edb_trace_time		__P((EdbContext, char *, ...));
int		edb_trace_to_syslog	__P((EdbContext, char *, ...));

int			edb_begin					__P((EdbContext, int));
int			edb_end					__P((EdbContext));
int			edb_pgsql_begin			__P((EdbContext, int));
int			edb_pgsql_end				__P((EdbContext));
int			edb_pgsql_pid				__P((EdbContext));
int			edb_pgsql_is_connected		__P((EdbContext));
int			edb_open					__P((EdbContext));
int			edb_setup_processes		__P((EdbContext, int));
int			edb_close					__P((EdbContext));
int			edb_listen					__P((EdbContext, utf8_t *));
int			edb_notify					__P((EdbContext, utf8_t *));
utf8_t	*	edb_accept_notify			__P((EdbContext));
eoid_t		edb_getage				__P((EdbContext));
int			edb_reldb_is_usable			__P((EdbContext));
int			edb_setdatabasespeaking		__P((EdbContext, char *));
int			edb_pgsql_setnonblocking		__P((EdbContext, int));
int			edb_pgsql_flush				__P((EdbContext, int));
int			edb_pgsql_command_exec		__P((EdbContext, utf8_t *));
Link			edb_pgsql_query			__P((EdbContext, utf8_t *, int *, int *));
typedef	int	(*EdbPgsqlQuery_callback_func_t)	__P((EdbContext, Link, void *));
int			edb_pgsql_query_callback		__P((EdbContext, utf8_t *, EdbPgsqlQuery_callback_func_t, void *));
Link			edb_pgsql_query_one_field	__P((EdbContext, utf8_t *, int *));
Link			edb_pgsql_query_one_tuple	__P((EdbContext, utf8_t *, int *));
utf8_t	*	edb_pgsql_query_one_text		__P((EdbContext, utf8_t *));
int			edb_pgsql_query_int			__P((EdbContext, utf8_t *));
utf8_t	*	edb_pgsql_query_text		__P((EdbContext, utf8_t *));
int			edb_pgsql_query_texts		__P((EdbContext, utf8_t ***, int *, utf8_t *));
int			edb_pgsql_query_integers		__P((EdbContext, int **, int *, utf8_t *));
int			edb_pgsql_seek_eid			__P((EdbContext, utf8_t *, char *, ...));
int			edb_EID_get				__P((EdbContext, utf8_t *, char *, ...));
int			edb_EIDs_get				__P((EdbContext, utf8_t *, int **, int *, char *, ...));
int			edb_get_eoids_from_eid		__P((EdbContext, utf8_t *, eid_t, int **, int *));
utf8_t	*	edb_create_latest_phrase		__P((EdbContext, utf8_t *, utf8_t *, utf8_t *, struct tm *, eid_t));
void			edb_vacuum				__P((EdbContext));

typedef	void	(*EdbProcessFunc)	__P((void *));
int			edb_process_setdatabasespeaking		__P((EdbContext, char *));
void			edb_process_initialize		__P((EdbContext));
void			edb_process_create			__P((EdbContext, EdbProcessFunc, void *));
void			edb_process_switch			__P((EdbContext, int));
void			edb_process_waitall			__P((EdbContext));
int			edb_process_number_of_active_processes	__P((EdbContext));
int			edb_process_number_of_usable_processes	__P((EdbContext));
int			edb_process_number_of_idle_processes		__P((EdbContext));

EdbCatalogue	edb_get_users			__P((EdbContext));

void		edb_delayed_free_mode		__P((EdbContext, int));
int		edb_delayed_free_exec		__P((EdbContext, int));

void		edb_sharable_text_mode		__P((EdbContext, int));

/* mode: 32bit */
#define	EDB_PRINT_MODE_LINK				0x00000001
#define	EDB_PRINT_MODE_ANCHOR			0x00000002
#define	EDB_PRINT_MODE_WWWLINK			0x00000004
#define	EDB_PRINT_MODE_COMPLETION		0x00000008
#define	EDB_PRINT_MODE_BLANKTARGET		0x00000010
#define	EDB_PRINT_MODE_CHECKMATCHED	0x00000020
#define	EDB_PRINT_MODE_CAPNOMAPTO		0x00000040

/* eid */
#define	EDB_INDEX_GET_ref	1
#define	EDB_INDEX_GET_own	2
#define	EDB_INDEX_GET_perm	3
#define	EDB_INDEX_GET_map	4
#define	EDB_INDEX_GET_fresh	5
#define	EDB_INDEX_GET_table	6
#define	EDB_INDEX_GET_concern	7

#define	EDB_INDEX_GET_catalogue_ref	11
#define	EDB_INDEX_GET_catalogue_own	12
#define	EDB_INDEX_GET_catalogue_perm	13
#define	EDB_INDEX_GET_catalogue_map	14

void	* edb_eid_list_store			__P((EdbContext, eid_t, int, void *));
void	* edb_eid_list_restore			__P((EdbContext, eid_t));
EdbIndexInfo	edb_eid_list_store_index		__P((EdbContext, eid_t, EdbIndexInfo));
EdbIndexInfo	edb_eid_list_restore_index		__P((EdbContext, eid_t));
eid_t	edb_eid_list_remove			__P((EdbContext, eid_t));
void	edb_eid_list_purge			__P((EdbContext));
void	edb_eid_clear_auth_check		__P((EdbContext));

Link	edb_eid_get_tuple_indexes		__P((EdbContext, ...));
int	edb_eid_get_number_of_tuples		__P((EdbContext, ...));
Link	edb_eid_get_tuple_indexes_from_table	__P((EdbContext, EdbTableInfo));
Link	edb_eid_get_authed_tuple_indexes_from_table	__P((EdbContext, EdbTableInfo));
int	edb_eid_get_number_of_available_tuples	__P((EdbContext, EdbTableInfo));
eid_t	edb_eid_isin_table			__P((EdbContext, eid_t));
eid_t	edb_eid_isin_table0			__P((EdbContext, eid_t));
int	edb_eid_referenced_table		__P((EdbContext, eid_t, int **));
Link	edb_eid_is_referenced_from_table	__P((EdbContext, EdbTableInfo, eid_t));
int	edb_eid_owned_table			__P((EdbContext, eid_t, int  **));
Link	edb_eid_is_owned_from_table	__P((EdbContext, EdbTableInfo, eid_t));
int	edb_eid_peid_table			__P((EdbContext, eid_t, int **));
Link	edb_eid_is_peid_from_table	__P((EdbContext, EdbTableInfo, eid_t));
int	edb_eid_mapto_table			__P((EdbContext, eid_t, int **));
Link	edb_eid_is_mapto_from_table	__P((EdbContext, EdbTableInfo, eid_t));
int	edb_eid_is_referenced		__P((EdbContext, eid_t));
int	edb_eid_is_mapped			__P((EdbContext, eid_t));
int	edb_eid_is_owner			__P((EdbContext, eid_t));
int	edb_eid_is_available			__P((EdbContext, eid_t));
int	edb_eid_tuple_is_available		__P((EdbContext, eid_t));

/* text */
int			edb_text_isAscii			__P((EdbContext, utf8_t *));
utf8_t	*	edb_text_encode			__P((EdbContext, utf8_t *));
utf8_t	*	edb_text_decode			__P((EdbContext, utf8_t *));
utf8_t	*	edb_mtext_concatenate		__P((EdbContext, eid_t, utf8_t *, utf8_t *, utf8_t *));
utf8_t	*	edb_mtext_concatenate_variant	__P((EdbContext, eid_t, utf8_t *, utf8_t *, utf8_t *));
void			edb_mtext_split				__P((EdbContext, eid_t *, utf8_t **, utf8_t **, utf8_t **, utf8_t *));
utf8_t	*	edb_text_for_pgsql			__P((EdbContext, utf8_t *));
utf8_t	*	edb_text_for_pgsql_literal		__P((EdbContext, utf8_t *));
utf8_t	*	edb_text_for_pgsql_variant	__P((EdbContext, utf8_t *));
utf8_t	*	edb_text_for_pgsql_literal_variant	__P((EdbContext, utf8_t *));
utf8_t	*	edb_text_for_pgsql_array		__P((EdbContext, utf8_t *));
int			edb_text_get_reference		__P((EdbContext, utf8_t *, utf8_t *));

typedef	int	(*EdbTextParseCallback)	__P((void *, utf8_t, utf8_t *));
int		edb_text_parse				__P((utf8_t **, EdbTextParseCallback, void *));
typedef	int	(*EdbTextUCS4ParseCallback)	__P((void *, ucs4_t, ucs4_t *));
int		edb_text_ucs4_parse			__P((ucs4_t **, EdbTextUCS4ParseCallback, void *));

int		edb_date_is_valid			__P((int, int, int));
int		edb_date_lastday			__P((int, int));
int		edb_date_kind				__P((int, int, int));

/* misc */
Link		edb_array_split				__P((utf8_t *));
char		*eid2strdup				__P((eid_t));
char		*eid2refstrdup				__P((eid_t));
eid_t		refstr2eid					__P((char *));
char		*strdup_null				__P((char *));
int		strcmp_null				__P((char *, char *));
int		str_is_valid				__P((char *));
void		edb_text_conv_initialize		__P((char *));
ucs4_t	* edb_text_conv				__P((EdbContext, ucs4_t *));
utf8_t	* edb_text_codename_to_html	__P((EdbContext, utf8_t *));
utf8_t	* text_codename_to_html		__P((utf8_t *));
utf8_t	* text_codename_to_tex		__P((utf8_t *));
utf8_t	* text_codename_to_text		__P((utf8_t *));
ucs4_t	* text_ucs4_codename_to_html	__P((ucs4_t *));
ucs4_t	* text_ucs4_codename_to_tex	__P((ucs4_t *));
ucs4_t	* text_ucs4_codename_to_text	__P((ucs4_t *));
ucs4_t	* text_ucs4_unicode_to_tex	__P((ucs4_t));

struct TEXT_CONVERSION {
	utf8_t	*utf8_name;	/* name of character \C{} */
	ucs4_t	*ucs4_name;	/* name of character \C{} */
	ucs4_t	*ucs4_src;	/* input characters */
	ucs4_t	*ucs4_d;		/* inner expression (unicode) */
	utf8_t	*utf8_d;		/* inner expression (unicode) */
	ucs4_t	*ucs4_tex;	/* tex string */
	utf8_t	*utf8_tex;		/* tex string */
	int		ucs4_dlen;
	int		count;
};
typedef	struct TEXT_CONVERSION	*TextConversion;

void		edb_text_conv_show_table	__P((EdbContext, FILE *));
void		edb_text_conv_list_callback	__P((EdbContext, int (*) __P((TextConversion, void *)), void *d));
void		edb_text_conv_dump_table	__P((EdbContext, char *, FILE *));
void		edb_text_conv_dump_table_for_variant	__P((FILE *));
void		edb_index_sort_bymtime		__P((EdbContext, Link));
void		pronounce_convert_order		__P((ucs4_t *));

#define	TEXT_VARIANT_CONVERSION
#if defined(TEXT_VARIANT_CONVERSION)
void		edb_text_variant_initialize		__P((char *));
void		edb_text_variant_dump		__P((FILE *));
void		edb_text_variant_list_callback	__P((int (*) __P((BiTree_obj_t, void *)), Doc));
utf8_t	*	edb_text_variant_conversion	__P((utf8_t *, int));
ucs4_t	edb_text_kana_variant_conversion	__P((ucs4_t));
#endif	/* TEXT_VARIANT_CONVERSION */

/* mail */
int	mail_send_notify			__P((char *, char *, char *, FILE *));

/* notify */
struct EDB_NOTIFY_MESSAGE {
	eid_t	recipient;
	Link	l_messages;
};
typedef	struct EDB_NOTIFY_MESSAGE	*EdbNotifyMessage;

extern	int	edb_notify_tuple_changes		__P((EdbContext, EdbCatalogue, EdbTupleInfo, EdbTupleInfo));
extern	int	edb_notify_tuple_create		__P((EdbContext, EdbCatalogue, EdbTupleInfo));
extern	int	edb_notify_tuple_delete		__P((EdbContext, EdbCatalogue, EdbTupleInfo));
extern	int	edb_notify_tuple_revive		__P((EdbContext, EdbCatalogue, EdbTupleInfo));
extern	int	edb_notify_send_messages	__P((EdbContext, EdbCatalogue, int));

#include	"edb/file.h"

/* time */
int		gettime				__P((void));
double	getcputime			__P((void));
char		*getdatestr			__P((char *));
int		gettoday_DATE			__P((void));
int		gethour				__P((void));
char		*gettodaystr			__P((char *));
char		*getdatestr_before_day	__P((char *));
char		*getdatestr_before_days	__P((char *, int));
char		*getdatestr_before_week	__P((char *));
char		*getdatestr_before_month	__P((char *));
char		*getdatestr_before_year	__P((char *));
char		*getdatestr_from_time	__P((char *, time_t));
int		getdays				__P((void));
int		getwday				__P((void));
char		*time2datestr			__P((char *, time_t));
time_t	datestr2time			__P((char *));
time_t	datestr2time000000		__P((char *));
time_t	time2time000000		__P((time_t));
int		time2wsec		__P((time_t));
int		time2ysec			__P((time_t));

#include	"edb/order.h"

int		edb_make_manual			__P((EdbContext, int, char *[], FILE *));

/* for malloc debug */
#define	_EDB_MALLOC_DEBUG_
#undef	_EDB_MALLOC_DEBUG_

#if defined(_EDB_MALLOC_DEBUG_)
void		malloc_debug_alloc	__P((void));
void		malloc_debug_free	__P((void));

void		*edb_memory_malloc	__P((size_t, char *, int));
void		edb_memory_free		__P((void *, char *, int));

#if 0
#define	Edb_malloc(siz)			edb_memory_malloc((siz), __FILE__, __LINE__)
#define	Edb_free(p)			edb_memory_free((p), __FILE__, __LINE__)
#else
#define	Edb_malloc(siz)			malloc((siz))
#define	Edb_free(p)			free((p))
#endif

#else
#define	Edb_malloc(siz)			malloc((siz))
#define	Edb_free(p)			free((p))
#endif

#endif	/* EDB_Inc_Defined */
