/*
	$Id: print.h,v 1.44 2012/04/03 03:51:40 alex Exp alex $
	print.h --- header file for print
*/

#ifndef	Print_Inc_Defined
#define	Print_Inc_Defined

struct EDB_COLUMN_PRINT {
	utf8_t	*xmlname;
	int		lang;
};
typedef	struct EDB_COLUMN_PRINT	*EdbColumnPrint;
EdbColumnPrint	edb_column_print_new	__P((utf8_t *));
void			edb_column_print_free	__P((EdbColumnPrint));

struct EDB_PRINT {
	EdbContext	ec;
	Doc			doc;

	struct EDB_PRINT_CONTEXT	*ct;
};
typedef	struct EDB_PRINT	*EdbPrint;
#define	EpContext(ep)		((ep)->ct)

#define	EP_PrintStyle_STANDARD		0	/* standard (default) */
#define	EP_PrintStyle_XML			1	/* XML */
#define	EP_PrintStyle_CSV			2	/* "Comma Separated Value", in fact "TAB Separeted Value" */
#define	EP_PrintStyle_ITEMIZE		3	/* itemize */
#define	EP_PrintStyle_CAPTION		4	/* caption */
#define	EP_PrintStyle_UTLF			5	/* UTLF */

#define	EP_PrintStyle_Variations		6	/* NOTICE: number of variations. Must be set the number of PrintStyle */


#define	PrintStyleIsSTANDARD(ep)	(PrintStyle((ep)) == PrintStyle_STANDARD)
#define	PrintStyleIsXML(ep)			(PrintStyle((ep)) == PrintStyle_XML)
#define	PrintStyleIsUTLF(ep)		(PrintStyle((ep)) == PrintStyle_UTLF)
#define	PrintStyleIsCSV(ep)			(PrintStyle((ep)) == PrintStyle_CSV)
#define	PrintStyleIsITEMIZE(ep)		(PrintStyle((ep)) == PrintStyle_ITEMIZE)
#define	PrintStyleIsCAPTION(ep)		(PrintStyle((ep)) == PrintStyle_CAPTION)

/* backward compatibility */
#define	PrintStyle_STANDARD	EP_PrintStyle_STANDARD
#define	PrintStyle_XML			EP_PrintStyle_XML
#define	PrintStyle_UTLF		EP_PrintStyle_UTLF
#define	PrintStyle_CSV			EP_PrintStyle_CSV
#define	PrintStyle_ITEMIZE		EP_PrintStyle_ITEMIZE
#define	PrintStyle_CAPTION		EP_PrintStyle_CAPTION

#define	PrintIsSTANDARD(ep)	PrintStyleIsSTANDARD(ep)
#define	PrintIsXML(ep)			PrintStyleIsXML(ep)
#define	PrintIsUTLF(ep)		PrintStyleIsUTLF(ep)
#define	PrintIsCSV(ep)			PrintStyleIsCSV(ep)
#define	PrintIsITEMIZE(ep)		PrintStyleIsITEMIZE(ep)
#define	PrintIsCAPTION(ep)		PrintStyleIsCAPTION(ep)

#define	EP_LANG_Auto		0		/* default */
#define	EP_LANG_English	1
#define	EP_LANG_Japanese	2

typedef	int	(*EdbPrintTCDFunc_t)	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
typedef	int	(*EdbPrintTupleFunc_t)	__P((EdbPrint, EdbTupleInfo));
typedef	int	(*EdbPrintTupleLangFunc_t)	__P((EdbPrint, EdbTupleInfo));

#define	EP_RECURSION_DEPTH	5

#define	EP_TCD_MODE_NORECURSION		0x00000001
#define	EP_TCD_MODE_ITALIC				0x00000002
#define	EP_TCD_MODE_BOLD				0x00000004
#define	EP_TCD_MODE_TT					0x00000008
#define	EP_TCD_MODE_ALERT				0x00000010

#define	EP_COLUMN_MODE_AT_ANYTIME		0x00000001
#define	EP_COLUMN_MODE_LIST_AND			0x00000002

struct EDB_PRINT_TYPE_INFO {
	int				typeID;
	EdbPrintTCDFunc_t	func[EP_PrintStyle_Variations];
};
typedef	struct EDB_PRINT_TYPE_INFO	*EdbPrintTypeInfo;

struct EDB_PRINT_TCD_INFO {
	char	*			xmlname;
	EdbPrintTCDFunc_t	func[EP_PrintStyle_Variations];
};
typedef	struct EDB_PRINT_TCD_INFO	*EdbPrintTCDInfo;

struct EDB_PRINT_TUPLE_INFO {
#if 0
	EdbTableInfo		ti;
#else
	eid_t				teid;
#endif
	BiTree			bt_print_tcd_info;
	EdbPrintTupleLangFunc_t	lang;
	EdbPrintTupleFunc_t	func[EP_PrintStyle_Variations];
};
typedef	struct EDB_PRINT_TUPLE_INFO	*EdbPrintTupleInfo;

struct EDB_PRINT_MODULE_INFO {
	char					*xmlname;
	EdbPrintTupleLangFunc_t	lang;
	EdbPrintTupleFunc_t		std;
	EdbPrintTupleFunc_t		xml;
	EdbPrintTupleFunc_t		csv;
	EdbPrintTupleFunc_t		item;
	EdbPrintTupleFunc_t		cap;
};

#if __GNUC__ > 2
#define	DECLARE_PRINT_MODULE(id, info)							\
struct EDB_PRINT_MODULE_INFO	* print_module_info_##id = &(info) ;		\
static char	*__print_module_info__ __attribute__((used)) = "EDB_PRINT_MODULE_INFO: print_module_info_" #id ;
#else
#define	DECLARE_PRINT_MODULE(id, info)							\
struct EDB_PRINT_MODULE_INFO	* print_module_info_##id = &(info) ;		\
static char	*__print_module_info__ = "EDB_PRINT_MODULE_INFO: print_module_info_" #id ;
#endif


struct EDB_PRINT_CONTEXT {
	struct EDB_PRINT_CONTEXT	*prev;

	int	style;
#define	PrintStyle(ep)			((ep)->ct->style)

	eid_t			tuple_ref_eoi;
	int			eoi_tobe_free;
	BiTree		bt_eoi;

	EdbPrintTupleFunc_t	print_tuple_prefix[EP_PrintStyle_Variations];
	int			print_tuple_tobe_free;
	BiTree		bt_print_tuple_info;
	EdbPrintTupleFunc_t	print_tuple_postfix[EP_PrintStyle_Variations];

	int			print_type_tobe_free;
	BiTree		bt_print_type_info;

#define	EP_PERSON_FULLNAME		0
#define	EP_PERSON_G_SURNAME	1
#define	EP_PERSON_SURNAME		2
	int			person_name_mode;

	EdbPrintTCDFunc_t	print_person_name[EP_PrintStyle_Variations];
	EdbPrintTCDFunc_t	print_page_number[EP_PrintStyle_Variations];
	EdbPrintTCDFunc_t	print_patent_number[EP_PrintStyle_Variations];

	int			lang;
#define	EpLang(ep)		((ep)->ct->lang)
#define	LangIsAuto(ep)		(EpLang(ep)==EP_LANG_Auto)
#define	LangIsEnglish(ep)	(EpLang(ep)==EP_LANG_English)
#define	LangIsJapanese(ep)	(EpLang(ep)==EP_LANG_Japanese)
	int			persist_lang;

#define	EP_TUPLE_LongFormat	0x00000001
#define	EP_TUPLE_TableName	0x00000002
#define	EP_TUPLE_OnlyAvail		0x00000004
#define	EP_TUPLE_ModifiedTime	0x00000008
#define	EP_TUPLE_Operator		0x00000010
#define	EP_TUPLE_RefColumn	0x00000020
#define	EP_TUPLE_PersonInfo	0x00000040
#define	EP_TUPLE_InTable		0x00000080
#define	EP_TUPLE_EditButton	0x00000100
#define	EP_TUPLE_CheckBox	0x00000200
#define	EP_TUPLE_SmallIcon	0x00000400
#define	EP_TUPLE_DeleteButton	0x00000800
#define	EP_TUPLE_ReviveButton	0x00001000
#define	EP_TUPLE_DuplicateButton	0x00002000
#define	EP_TUPLE_EdbClientButton	0x00004000
#define	EP_TUPLE_BrowseButton	0x00008000
#define	EP_TUPLE_EdbAssistanceButton	0x00010000
	int			mode;
	int			tcd_mode;
	int			column_mode;
	int			tcd_idx;			/* index of datum */
	char			*tcd_idx_prefix;		/* prefix of datum */
	char			*tuple_tag_prefix;
	char			*tuple_tag_postfix;

	/* control */
	int	noalert;
	int	nocolor;
	int	nounderline;
	int	noeidprefix;
	int	noauthor;

	/* alert level */
	int	alert_level;

	/* status */
#define	EP_STATUS_NORMAL	0x0
#define	EP_STATUS_WARNING	0x1
#define	EP_STATUS_ALERT		0x2
	int	status;
#define	EpStatus(ep)	((ep)->ct->status)

	void	*user_data;	/* user data */
#define	EpUserData(ep)	((ep)->ct->user_data)
};


#define	EP_ALERT_COLOR	"Red"
#define	EP_WARNING_COLOR	"Fuchsia"

#define	EP_LaTeX_PREFIX	"edb"

EdbPrint	edb_print_new			__P((EdbContext, Doc, int));
EdbPrint	edb_print_dup			__P((EdbPrint));
void		edb_print_free			__P((EdbPrint));

void		edb_print_push			__P((EdbPrint));
void		edb_print_pop			__P((EdbPrint));

int		edb_print_tuple_prefix	__P((EdbPrint, EdbTupleInfo));
int		edb_print_tuple_postfix	__P((EdbPrint, EdbTupleInfo));
int		edb_print_tuple_prefix_register_func		__P((EdbPrint, int, EdbPrintTupleFunc_t));
int		edb_print_tuple_postfix_register_func	__P((EdbPrint, int, EdbPrintTupleFunc_t));
int		edb_print_tuple_caption	__P((EdbPrint, EdbTupleInfo, char *, ...));
int		edb_print_tuple			__P((EdbPrint, EdbTupleInfo));
int		edb_print_tuple_language	__P((EdbPrint, EdbTupleInfo));
int		edb_print_tcd			__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_tcd_prefix		__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_tcd_postfix	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_type			__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));

typedef	int	(*EdbPrintTupleColumnCallbackFunc_t)	__P((EdbPrint, EdbTupleInfo, EdbDatum));

int		edb_print_tuple_column			__P((EdbPrint, EdbTupleInfo, char *, char *, EdbTupleColumn, EdbPrintTupleColumnCallbackFunc_t, char *, char *, char *, int));
int		edb_print_tuple_named_column		__P((EdbPrint ep, EdbTupleInfo, EdbDatum, char *, char *, char *, EdbPrintTupleColumnCallbackFunc_t, char *, char *, char *, int));

void		edb_print_student_list			__P((EdbContext, utf8_t *, int, char *, char *, int, Doc));
void		edb_print_student_info			__P((EdbContext, StudentInfo, int, Doc));

int		edb_print_type_DATE	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_type_DATE_YMD	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_type_DATE_YM	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_type_DATE_Y	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));

int		edb_print_patent_number	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_pmodel_number	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));

void		edb_print_eoi_clear		__P((EdbPrint));
int		edb_print_eoi_check		__P((EdbPrint, eid_t));
eid_t		edb_print_eoi_add		__P((EdbPrint, eid_t));

int		edb_print_tuple_register_func	__P((EdbPrint, char *, int, EdbPrintTupleFunc_t));
int		edb_print_tuple_register_language_func	__P((EdbPrint, char *, EdbPrintTupleLangFunc_t));
int		edb_print_tcd_register_func	__P((EdbPrint, char *, int, EdbPrintTCDFunc_t));
int		edb_print_type_register_func	__P((EdbPrint, int, int, EdbPrintTCDFunc_t));

int		edb_book_print_AND		__P((EdbPrint));
void	edb_book_print_delimiter		__P((EdbPrint, int, char *));
int	edb_book_print_column		__P((EdbPrint, EdbTupleInfo, utf8_t *, int, int, int, int));
int	edb_book_print_named_column	__P((EdbPrint, EdbTupleInfo, char *, char *, char *, char *, char *, char *, int, int));
int	edb_book_print_named_column_AND	__P((EdbPrint, EdbTupleInfo, char *, char *, char *, char *, char *, char *, int, int));
int	edb_book_print_caption		__P((EdbPrint, EdbTupleInfo));
int	edb_book_print_tuple			__P((EdbPrint, EdbTupleInfo, Link));

int	edb_print_tuple_XML			__P((EdbPrint, EdbTupleInfo));
int	edb_print_tuple_UTLF		__P((EdbPrint, EdbTupleInfo));

void		edb_print_table_information		__P((EdbPrint, EdbTableInfo, int));
void		edb_print_table_in_xml			__P((EdbPrint, EdbTableInfo, int));
void		edb_print_table_definition			__P((EdbPrint, EdbTableInfo));

void		edb_print_tuple_datum_permission	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
void		edb_print_tuple_column_head		__P((EdbPrint));
void		edb_print_tuple_column_foot		__P((EdbPrint));
void		edb_print_tuple_column_information	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum, Link, int));
void		edb_print_tuple_column_datum		__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum, int));
void		edb_print_tuple_information		__P((EdbPrint, EdbTupleInfo, int));
void		edb_print_tuple_in_xml			__P((EdbPrint, EdbTupleInfo, int));
void		edb_print_tuple_diff_information	__P((EdbPrint, EdbTupleInfo, EdbTupleInfo, int));
void		edb_print_tuple_diff_column		__P((EdbPrint, EdbTupleInfo, EdbTupleInfo, int));

#if defined(TUIR_COOPERATION)
char	*	tuir_create_url					__P((EdbContext, EdbTupleInfo));
void		tuir_show_iframe				__P((EdbContext, char *, EdbTupleInfo, char *, Doc));
#endif /* TUIR_COOPERATION */

int		edb_print_issn_information		__P((EdbContext, Doc, char *));
int		edb_print_issn_information_body	__P((EdbContext, Doc, char *));
int		edb_print_magazine_information		__P((EdbContext, Doc, eid_t, char *, int));

#define	EP_WARNING	1
#define	EP_ALERT		2
int		edb_print_alert					__P((EdbPrint, int, char *, ...));
int		edb_print_alert_column			__P((EdbPrint, int, EdbColumnInfo));

int		edb_print_punctuation_puts		__P((EdbPrint, char *));

void		edb_print_type_initialize			__P((EdbPrint));
BiTree	edb_print_tuple_duplicate_info		__P((BiTree));
BiTree	edb_print_type_duplicate_info		__P((BiTree));
BiTree	edb_print_tcd_duplicate_info		__P((BiTree));

int		edb_print_type_EMAIL_in_HTML_avoid_SPAM	__P((EdbPrint, EdbTupleInfo, EdbTupleColumn, EdbDatum));
int		edb_print_emailaddress_in_HTML_avoid_SPAM	__P((Doc, char *));

#endif	/* Print_Inc_Defined */


