/*
	$Id: table.h,v 1.38 2012/04/03 03:51:13 alex Exp alex $
	table.h --- header file for table
*/

#ifndef	Table_Inc_Defined
#define	Table_Inc_Defined

#define	EDB_TABLE_TABLE			"edb_table"

#include	"edb/eid.h"
#include	"edb/base.h"

#define	TABLE_MAKE_ALL_TEXT
#if defined(TABLE_MAKE_ALL_TEXT)
#define	EDB_COLUMN_CAP_TEXT		"cap_text"
#define	EDB_COLUMN_CAP_VARIANT	"cap_variant"
#define	EDB_COLUMN_CAP_EID		"cap_eid"

#define	EDB_COLUMN_ALL_TEXT		"all_text"
#define	EDB_COLUMN_ALL_VARIANT	"all_variant"
#define	EDB_COLUMN_ALL_EID		"all_eid"
#endif

#define	EDB_COLUMN_L_EID		"l_eid"
#define	EDB_COLUMN_L_PEID		"l_peid"
#define	EDB_COLUMN_RELDB_INDEX	"r_idx"

struct EDB_MAPLOOKUP {
	utf8_t	*lookup;
	int		parsed;
	eid_t		teid;
	Link		l_caption_ci;
	Link		l_candidate_condition;
};
typedef	struct EDB_MAPLOOKUP	*EdbMapLookup;

struct EDB_COLUMN_INFO {
	struct EDB_TABLE_INFO		*ti;		/* belonging table */
	struct EDB_COLUMN_INFO	*super;	/* super column */
	Link	l_child;						/* children column */
	int	columns;						/* number of columns under this column (include own) */
	int	idx;							/* index in table->column */
	int	col;

	utf8_t	*reldb_sqlname;					/* sqlname of reldb */
#if 0
	int	reldb_exist;					/* existence of reldb */
#endif

	utf8_t	*xmlname;						/* element name in XML */
	utf8_t	*sqlname;						/* column name in SQL */
	utf8_t	*sqlname_variant;				/* column name in SQL (variant) */
	utf8_t	*sqlname_eid;					/* column name in SQL (EID) */

#if 0
	char	*type;						/* type name in EDB */
#endif
	EdbCTypeInfo	ctype;				/* column dependent type info */
#define	ColumnCType(c)			((c)->ctype)
#define	ColumnTypeName(c)			(ColumnCType((c))->name)
#define	ColumnTypeInfo(c)			(ColumnCType((c))->info)
#define	ColumnTypeMinimum(c)		(ColumnCType((c))->v_minimum)
#define	ColumnTypeMaximum(c)		(ColumnCType((c))->v_maximum)
#define	ColumnTypeDefault(c)		(ColumnCType((c))->v_default)
#define	ColumnTypeCryptography(c)	(ColumnCType((c))->cryptography)
	EdbTypeInfo	typeInfo;				/* type info. in EDB */

	int	authorize;						/* authorized reference */
	int	classify;						/* use classifing */
#define	CLASSIFY_PRIMARY		1
#define	CLASSIFY_SECONDARY	2
	int	alias;							/* use classifing */
	int	chronological;					/* chronological column */
	int	conductor;						/* conductor */
	int	obsoleted;						/* obsoleted column */
	int	exclusive;						/* exclusive */
	int	hierarchyparent;				/* use as hierarchy parent */
	int	multilingualcaption;				/* multilingual caption */
	int	personname;					/* person's name */
	int	sometimesStudent;				/* sometimes student */
	int	pagenumber;					/* page number */
	int	patentnumber;					/* patent number */
	int	canbeNothing;					/* maybe nothing */
	int	elements;						/* number of elements */
	char	*elasticity;					/* elasticity of elements */
	int	elements_base;					/* base number of elements */
	int	elements_delta;					/* incremental number of elements */
#define	USAGE_UNUSED		-1
#define	USAGE_UNKNOWN		0
#define	USAGE_OPTIONAL		1
#define	USAGE_PREFERRED		2
#define	USAGE_REQUIRED		3
	int	usage;						/* column built-in usage */
	int	conductor_usage;				/* conductor's usage mode */
	Link	l_maplookup;					/* map lookup strings */
	struct EDB_PERMISSION	perm_tuple;
	struct EDB_MTEXT	name;
	struct EDB_BTEXT	desc;
	struct EDB_BTEXT	explain;
#define	EDB_CAPTION_NONE	0
#define	EDB_CAPTION_SHORT	1
#define	EDB_CAPTION_LONG	2
	struct EDB_BTEXT	prefix;			/* prefix (ex. Vol., No.) */
	struct EDB_BTEXT	suffix;			/* suffix (ex. 巻, 号) */
	int	caption;
#define	EDB_ORDER_MAX		10
	int	order;
	int	size;
};
typedef	struct EDB_COLUMN_INFO	*EdbColumnInfo;

#include	"edb/caption.h"

struct EDB_TABLE_INFO {
	struct EDB_BASE_INFO base;
	utf8_t	*xmlname;
	utf8_t	*sqlname;
	struct EDB_MTEXT	name;
	struct EDB_BTEXT	desc;
	struct EDB_BTEXT	explain;
	Link	l_maplookup;
	/* default auth level */
	struct EDB_PERMISSION	perm_tuple;
	struct EDB_PERMISSION	perm_column;
	int	auxiliary;				/* auxiliary table */
	int	hierarchical;			/* hierarchical structured information */
	int	censorship;			/* need censorship */
	int	chronological;			/* chronological information */
	char	*module;				/* module name */
	int	ncol;					/* number of columns */
	Link	l_column;				/* linked top level column, multicolumn */
	EdbColumnInfo	*column;		/* column primary info */

	EdbCatalogue	ca_all_tuples;	/* all tuples */
	Link			l_all_indexes;	/* indexes of all tuples */

	int	module_checked;
	void	*module_handle;
	Caption	(*tuple_make_caption_func)();
	int		(*tuple_check_updatable)();
	int		(*tuple_print_additional_information)();
#if defined(EDB_MANAGE_PKI)
	void	*	(*tuple_certificate_request_info)();
#endif
	int		(*tuple_check_writable_by)();
	Link		l_conductors;			/* list of conductor columns */
};
typedef	struct EDB_TABLE_INFO	*EdbTableInfo;

#define	TABLE_Base(t)		(&((t)->base))
#define	TABLE_EID(t)		BaseEID(TABLE_Base((t)))
#define	TABLE_EOID(t)		BaseEOID(TABLE_Base((t)))
#define	TABLE_Mapto(t)	BaseMapto(TABLE_Base((t)))
#define	TABLE_Parent(t)	BaseParent(TABLE_Base((t)))
#define	TABLE_Mtime(t)	BaseMtime(TABLE_Base((t)))
#define	TABLE_Operator(t)	BaseOperator(TABLE_Base((t)))
#define	TABLE_Avail(t)		BaseAvail(TABLE_Base((t)))
#define	TABLE_Owner(t)	BaseOwner(TABLE_Base((t)))
#define	TABLE_Read(t)		BaseRead(TABLE_Base((t)))
#define	TABLE_Write(t)		BaseWrite(TABLE_Base((t)))
#define	TABLE_Delete(t)	BaseDelete(TABLE_Base((t)))
#define	TABLE_TupleCreate(t)	((t)->perm_tuple.p_create)
#define	TABLE_TupleRead(t)		((t)->perm_tuple.p_read)
#define	TABLE_TupleWrite(t)		((t)->perm_tuple.p_write)
#define	TABLE_TupleDelete(t)	((t)->perm_tuple.p_delete)
#define	TABLE_ColumnCreate(t)	((t)->perm_column.p_create)
#define	TABLE_ColumnRead(t)	((t)->perm_column.p_read)
#define	TABLE_ColumnWrite(t)	((t)->perm_column.p_write)
#define	TABLE_ColumnDelete(t)	((t)->perm_column.p_delete)

#define	TABLE_Name_en(t)	((t)->name.english ? (t)->name.english : (utf8_t *)"")
#define	TABLE_Name_ja(t)	((t)->name.japanese ? (t)->name.japanese : (utf8_t *)"")
#define	TABLE_Name_pr(t)	((t)->name.pronounce ? (t)->name.pronounce : (utf8_t *)"")

#define	TABLE_Desc_en(t)	((t)->desc.english ? (t)->desc.english : (utf8_t *)"")
#define	TABLE_Desc_ja(t)	((t)->desc.japanese ? (t)->desc.japanese : (utf8_t *)"")

#define	TABLE_Explain_en(t)	((t)->explain.english ? (t)->explain.english : (utf8_t *)"")
#define	TABLE_Explain_ja(t)	((t)->explain.japanese ? (t)->explain.japanese : (utf8_t *)"")

#define	TABLE_XN(t)			((t)->xmlname)
#define	TABLE_SN(t)			((t)->sqlname)

#define	ColumnTupleRead(c)		((c)->perm_tuple.p_read)
#define	ColumnTupleCreate(c)	((c)->perm_tuple.p_create)
#define	ColumnTupleWrite(c)		((c)->perm_tuple.p_write)
#define	ColumnTupleDelete(c)	((c)->perm_tuple.p_delete)
#define	ColumnTable(c)			((c)->ti)
#define	ColumnSuper(c)		((c)->super)
#define	ColumnUsage(c)		((c)->usage)
#define	ColumnConductorUsage(c)		((c)->conductor_usage)

#define	ColumnCanbeNothing(c)		((c)->canbeNothing)

#define	ColumnXN(c)			((c)->xmlname)
#define	ColumnSN(c)			((c)->sqlname)
#define	ColumnSN_VARIANT(c)	((c)->sqlname_variant)
#define	ColumnSN_EID(c)		((c)->sqlname_eid)
#define	ColumnRdbSN(c)		((c)->reldb_sqlname)

#define	ColumnName_en(c)	((c)->name.english ? (c)->name.english : (utf8_t *)"")
#define	ColumnName_ja(c)	((c)->name.japanese ? (c)->name.japanese : (utf8_t *)"")
#define	ColumnName_pr(c)	((c)->name.pronounce ? (c)->name.pronounce : (utf8_t *)"")

#define	ColumnDesc_en(c)	((c)->desc.english ? (c)->desc.english : (utf8_t *)"")
#define	ColumnDesc_ja(c)	((c)->desc.japanese ? (c)->desc.japanese : (utf8_t *)"")

#define	ColumnExplain_en(c)	((c)->explain.english ? (c)->explain.english : (utf8_t *)"")
#define	ColumnExplain_ja(c)	((c)->explain.japanese ? (c)->explain.japanese : (utf8_t *)"")

#define	ColumnPrefix_en(c)	((c)->prefix.english ? (c)->prefix.english : (utf8_t *)"")
#define	ColumnPrefix_ja(c)	((c)->prefix.japanese ? (c)->prefix.japanese : (utf8_t *)"")

#define	ColumnSuffix_en(c)	((c)->suffix.english ? (c)->suffix.english : (utf8_t *)"")
#define	ColumnSuffix_ja(c)	((c)->suffix.japanese ? (c)->suffix.japanese : (utf8_t *)"")

/* table */
EdbTableInfo	edb_table_new					__P((EdbContext));
EdbTableInfo	edb_table_get					__P((EdbContext, eid_t));
EdbTableInfo	edb_table_get_by_name			__P((EdbContext, utf8_t *));
EdbTableInfo	edb_table_parse_xml				__P((EdbContext, utf8_t *));
utf8_t	*	edb_table_make_xml				__P((EdbContext, EdbTableInfo));
EdbTableInfo	edb_table_get_XMLDB			__P((EdbContext, eid_t, eoid_t));
eoid_t		edb_table_put_XMLDB			__P((EdbContext, EdbTableInfo));
int			edb_table_delete_XMLDB			__P((EdbContext, eid_t, eoid_t));
int			edb_table_column_reldb_elements	__P((EdbContext, EdbTableInfo, EdbColumnInfo, EdbColumnInfo));
int			edb_table_column_xml_elements	__P((EdbContext, EdbTableInfo, EdbColumnInfo, EdbColumnInfo));
EdbTableInfo	edb_table_get_byEOID			__P((EdbContext, eoid_t));
eid_t			edb_table_put					__P((EdbContext, EdbTableInfo, eid_t));
eoid_t		edb_table_put_reldb				__P((EdbContext, EdbTableInfo));
int			edb_table_check				__P((EdbContext, EdbTableInfo));
eid_t			edb_table_delete				__P((EdbContext, eid_t));
EdbTableInfo	edb_table_dup					__P((EdbContext, EdbTableInfo));
void			edb_table_free					__P((EdbContext, EdbTableInfo));
eid_t			edb_table_sqlname_to_eid			__P((EdbContext, utf8_t *));
eid_t			edb_table_xmlname_to_eid		__P((EdbContext, utf8_t *));
eid_t			edb_table_name_to_eid			__P((EdbContext, utf8_t *));
utf8_t	*	edb_table_make_order			__P((EdbContext, EdbTableInfo, utf8_t *));
Link			edb_table_get_eid_of_tuples		__P((EdbContext, EdbTableInfo, char *, ...));
Link			edb_table_get_eid_of_available_tuples	__P((EdbContext, EdbTableInfo, char *, ...));
void			edb_table_prefetch_tuples			__P((EdbContext, EdbTableInfo));
Link			edb_table_get_tuples				__P((EdbContext, EdbTableInfo, char *, ...));
Link			edb_table_get_ordered_tuples		__P((EdbContext, EdbTableInfo, Link, char *, ...));
Link			edb_table_get_tuples_for_look		__P((EdbContext, EdbTableInfo, Link, char *, ...));
Link			edb_table_get_tuples_use_index	__P((EdbContext, EdbTableInfo, char *, ...));
#if 0
int			edb_table_put_column			__P((EdbContext, EdbTableInfo, EdbColumnInfo));
int			edb_table_create_column			__P((EdbContext, EdbTableInfo, int, utf8_t **));
#endif
int			edb_table_tname_to_eid			__P((EdbContext, utf8_t *));
int			edb_table_create_reldb			__P((EdbContext, EdbTableInfo, int));

int			edb_table_reldb_will_be_changed	__P((EdbContext, EdbTableInfo));

EdbColumnInfo	edb_table_seek_column			__P((EdbContext, EdbTableInfo, utf8_t *));
EdbColumnInfo	edb_table_seek_column_by_name	__P((EdbContext, EdbTableInfo, utf8_t *));
EdbColumnInfo	edb_table_seek_column_by_xmlname	__P((EdbContext, EdbTableInfo, utf8_t *));
EdbColumnInfo	edb_table_seek_column_by_sqlname	__P((EdbContext, EdbTableInfo, utf8_t *));
EdbColumnInfo	edb_table_seek_picture_column		__P((EdbContext, EdbTableInfo));

Caption		edb_table_make_caption			__P((EdbContext, EdbTableInfo, int));
Link			edb_table_get_eid_of_tables		__P((EdbContext));
Link			edb_table_get_tables				__P((EdbContext));
Link			edb_table_search_tuples_byText	__P((EdbContext, EdbTableInfo, utf8_t *, int));
Link			edb_table_lookup_tuples_byText	__P((EdbContext, EdbTableInfo, EdbColumnInfo, utf8_t *));

/* maplookup */
EdbMapLookup	edb_maplookup_new				__P((EdbContext, utf8_t *));
void			edb_maplookup_free				__P((EdbContext, EdbMapLookup));
void			edb_maplookup_parse			__P((EdbContext, EdbMapLookup));
eid_t			edb_maplookup_table_eid			__P((EdbContext, EdbMapLookup));
void			edb_table_parse_maplookup		__P((EdbContext, EdbTableInfo));

int			edb_table_is_hierarchical			__P((EdbContext, EdbTableInfo));
EdbColumnInfo	edb_table_hierarchy_column		__P((EdbContext, EdbTableInfo));

int			edb_table_is_chronological			__P((EdbContext, EdbTableInfo));
int			edb_table_has_conductor			__P((EdbContext, EdbTableInfo));
Link			edb_table_get_conductors			__P((EdbContext, EdbTableInfo));

utf8_t *		edb_table_get_column_minimum	__P((EdbContext, EdbTableInfo, EdbColumnInfo, utf8_t *));
utf8_t *		edb_table_get_column_maximum	__P((EdbContext, EdbTableInfo, EdbColumnInfo, utf8_t *));
utf8_t *		edb_table_get_date2_minimum	__P((EdbContext, EdbTableInfo, EdbColumnInfo, utf8_t *));
utf8_t *		edb_table_get_date2_maximum	__P((EdbContext, EdbTableInfo, EdbColumnInfo, utf8_t *));
int			edb_table_get_number_of_tuples	__P((EdbContext, EdbTableInfo, char *, ...));
int			edb_table_get_number_of_available_tuples	__P((EdbContext, EdbTableInfo, char *, ...));

EdbColumnInfo	edb_table_get_date_column		__P((EdbContext, EdbTableInfo));

Link			edb_table_get_all_tables			__P((EdbContext));

struct	EDB_TABLE_CB_DATA	{
	void	*user_data;
};
typedef	struct EDB_TABLE_CB_DATA	*EdbTableCBData;

typedef	int	(*TableColumn_callback_func_t) __P((EdbContext, EdbTableInfo, int, EdbColumnInfo, EdbTableCBData));

void			edb_table_column_callback		__P((EdbContext, EdbTableInfo, EdbColumnInfo, TableColumn_callback_func_t, void *));

int			edb_table_reldb_exist			__P((EdbContext, utf8_t *));

int			edb_table_module_load			__P((EdbContext, EdbTableInfo));

int			edb_table_tuple_caption_size		__P((EdbContext, EdbTableInfo, int));

#define	CXN_STUDENT	"studentcode"

#define	TN_STUDENT	"t_student"
struct STUDENT_INFO {
	utf8_t	*v_id;
	utf8_t	*v_sid;
	utf8_t	*v_code;
	utf8_t	*v_name_ja;
	utf8_t	*v_name_en;
	utf8_t	*v_name_pr;
	int		v_enter;
	int		v_leave;
	utf8_t	*v_faculty;
	utf8_t	*v_department;
	utf8_t	*v_division;
	utf8_t	*v_title;

	int		ref_date;
};
typedef	struct STUDENT_INFO	*StudentInfo;

int		edb_table_column_is_studentcode	__P((EdbContext, EdbColumnInfo));
Link		edb_table_get_student_info		__P((EdbContext, utf8_t*));
void		edb_table_free_student_info		__P((StudentInfo));
char	*	edb_table_create_student_dict		__P((EdbContext, StudentInfo));
Link		edb_table_search_students_byText	__P((EdbContext, utf8_t *, int, int));
Link		edb_table_search_students_byDatum	__P((EdbContext, EdbDatum, int));

#endif	/* Table_Inc_Defined */

