/*
	$Id: tuple.h,v 1.41 2011/04/18 09:45:28 alex Exp $
	tuple.h --- header file for tuple
*/

#ifndef	Tuple_Inc_Defined
#define	Tuple_Inc_Defined

#include	"edb/eid.h"
#include	"edb/base.h"
#include	"edb/table.h"
#include	"edb/caption.h"
#include	"edb/xml.h"

struct EDB_TUPLE_COLUMN {
	void *super;	/* EdbTupleInfo super_tuple; or EdbDatum super_datum; */
	EdbColumnInfo	ci;
	struct EDB_DATUM	*datum;	/* EdbDatum */
	int	mapped;
	int	nothing:1;	// assert to nothing
};
typedef	struct EDB_TUPLE_COLUMN	*EdbTupleColumn;

struct EDB_TUPLE_INFO {
	struct EDB_BASE_INFO base;
	EdbTableInfo	ti;
	Link	l_column;

	int8_t	tuple_mapped;
	int8_t	tuple_perm_checked;
	int8_t	tuple_perm_eliminated;
//	eid_t		tuple_auth_check;

	/* location */
#define	TUPLE_LOCATION_UNKNOWN	0	/* unknown */
#define	TUPLE_LOCATION_INSIDE	1	/* inside of my organization */
#define	TUPLE_LOCATION_OUTSIDE	2	/* outside of my organization */
	int8_t	tuple_location;

	/* auth level */
	int8_t	tuple_auth_level_checked;
	int8_t	tuple_auth_user;
	int8_t	tuple_auth_personnel;
	int8_t	tuple_auth_teacher;
	/* expand */
	int8_t	expand_checked;
	Link	expand_map;
	EdbCatalogue	ca_expand;

	/* history */
	int		number_of_histories;

	/* conductor */
	int8_t	conductor_checked;		/* conductor lists were checked */
	int8_t	conductor_found;		/* conductor is found */
	Link	l_required_cxn;				/* required columns */
	Link	l_preferred_cxn;			/* preferred columns */
	Link	l_optional_cxn;				/* optional columns */
	Link	l_unused_cxn;				/* unused columns */
};
typedef	struct EDB_TUPLE_INFO	*EdbTupleInfo;

#define	TupleBase(t)		(&((t)->base))
#define	TupleEID(t)		BaseEID(TupleBase((t)))
#define	TupleEOID(t)		BaseEOID(TupleBase((t)))
#define	TupleMapto(t)		BaseMapto(TupleBase((t)))
#define	TupleParent(t)		BaseParent(TupleBase((t)))
#define	TupleMtime(t)		BaseMtime(TupleBase((t)))
#define	TupleOperator(t)	BaseOperator(TupleBase((t)))
#define	TupleAvail(t)		BaseAvail(TupleBase((t)))
#define	TupleDateFrom(t)	BaseDateFrom(TupleBase((t)))
#define	TupleDateTo(t)		BaseDateTo(TupleBase((t)))
#define	TupleOwner(t)		BaseOwner(TupleBase((t)))
#define	TupleRead(t)		BaseRead(TupleBase((t)))
#define	TupleWrite(t)		BaseWrite(TupleBase((t)))
#define	TupleDelete(t)		BaseDelete(TupleBase((t)))
#define	TupleTable(t)		((t)->ti)
#define	TupleXN(t)		(TABLE_XN(TupleTable(t)))
#define	TupleName_en(t)	(TABLE_Name_en(TupleTable(t)))
#define	TupleName_ja(t)	(TABLE_Name_ja(TupleTable(t)))
#define	TupleName_pr(t)	(TABLE_Name_pr(TupleTable(t)))


#define	ForTupleColumn(l, t)	ForLink(l, (t)->l_column)

/* tuple */
#define	EDB_TUPLE_GET_template_table	1
#define	EDB_TUPLE_GET_permitted		2
#define	EDB_TUPLE_GET_without_mapped	3
#define	EDB_TUPLE_GET_eoid			4
#define	EDB_TUPLE_GET_index			5
EdbTupleInfo	edb_tuple_new				__P((EdbContext, EdbTableInfo));
void			edb_tuple_free				__P((EdbContext, EdbTupleInfo));
void			edb_tuple_free_exec			__P((EdbContext, EdbTupleInfo));

EdbTupleColumn	edb_tuple_seek_column			__P((EdbContext, EdbTupleInfo, utf8_t *));
EdbTupleColumn	edb_tuple_seek_column_by_name	__P((EdbContext, EdbTupleInfo, utf8_t *));
EdbTupleColumn	edb_tuple_seek_column_by_sqlname	__P((EdbContext, EdbTupleInfo, utf8_t *));
EdbTupleColumn	edb_tuple_seek_column_by_xmlname	__P((EdbContext, EdbTupleInfo, utf8_t *));
EdbTupleColumn	edb_tuple_seek_TC_by_xmlname	__P((EdbContext, EdbTupleInfo, EdbTupleColumn, EdbDatum, utf8_t *));
EdbTupleColumn	edb_tuple_seek_picture_column		__P((EdbContext, EdbTupleInfo));

EdbTupleColumn	edb_tuple_column_new			__P((EdbContext, EdbTupleInfo, EdbDatum, EdbColumnInfo));
void				edb_tuple_column_free			__P((EdbContext, EdbTupleColumn));
EdbDatum			edb_tuple_TCD_dup				__P((EdbContext, EdbTupleInfo, EdbTupleColumn, EdbDatum));
EdbTupleColumn	edb_tuple_column_dup			__P((EdbContext, EdbTupleInfo, EdbDatum, EdbTupleColumn));

int			edb_tuple_check			__P((EdbContext, EdbTupleInfo));

#define	EdbTupleCheckUpdatable_P		__P((EdbContext, EdbTupleInfo))
int			edb_tuple_check_updatable	EdbTupleCheckUpdatable_P;
#define	EdbTupleCheckWritableBy_P		__P((EdbContext, EdbTupleInfo, eid_t))
int			edb_tuple_check_writable_by	EdbTupleCheckWritableBy_P;

int			edb_tuple_adjust			__P((EdbContext, EdbTupleInfo, int));
EdbTupleInfo	edb_tuple_get				__P((EdbContext, eid_t, ...));
void			edb_tuple_get_mapto			__P((EdbContext, EdbTupleInfo));
void			edb_tuple_import			__P((EdbContext, EdbTupleInfo, eid_t));
void			edb_tuple_import_column		__P((EdbContext, EdbTupleInfo, EdbTupleColumn));
EdbTupleInfo	edb_tuple_get_byEOID		__P((EdbContext, eoid_t));
EdbTupleInfo	edb_tuple_parse_xml			__P((EdbContext, EdbTableInfo, utf8_t *));
#define	TUPLE_MAKE_XML_COMPLETE		0x00000001
utf8_t	*	edb_tuple_make_xml			__P((EdbContext, EdbTupleInfo, int));
utf8_t	*	edb_tuple_datum_make_xml	__P((EdbContext, EdbTupleInfo, Link, int));
EdbTupleInfo	edb_tuple_duplicate			__P((EdbContext, EdbTupleInfo));
EdbTupleInfo	edb_tuple_get_XMLDB		__P((EdbContext, EdbTableInfo, eid_t, eoid_t));
Link			edb_tuple_get_XMLDBs		__P((EdbContext, EdbTableInfo));
int			edb_tuple_get_teid_XMLDB	__P((EdbContext, eid_t, eoid_t));
eoid_t		edb_tuple_put_XMLDB		__P((EdbContext, EdbTupleInfo));
eoid_t		edb_tuple_put_cXMLDB		__P((EdbContext, EdbTupleInfo));
int			edb_tuple_delete_XMLDB		__P((EdbContext, eid_t, eoid_t));
void			edb_tuple_get_ref_eid			__P((EdbContext, EdbTupleInfo, Link *, Link *, BiTree *));
EdbIndexInfo	edb_tuple_make_eid_index			__P((EdbContext, EdbTupleInfo));
int			edb_tuple_check_reference		__P((EdbContext, EdbTupleInfo));
void			edb_tuple_check_permission		__P((EdbContext, EdbTupleInfo));
eoid_t		edb_tuple_put					__P((EdbContext, EdbContext, EdbTupleInfo, int));
eoid_t		edb_tuple_put_reldb				__P((EdbContext, EdbTupleInfo, int));
eoid_t		edb_tuple_delete				__P((EdbContext, EdbContext, eid_t));
void			edb_tuple_delete_reldb			__P((EdbContext, EdbTableInfo, eid_t));
int			edb_tuple_erasable				__P((EdbContext, eid_t));
Caption		edb_tuple_make_caption			__P((EdbContext, EdbTupleInfo, int));

Caption		edb_datum_make_caption			__P((EdbContext, EdbDatum, int));

void			edb_tuple_wait_propagation		__P((EdbContext, eoid_t, long));

#if defined(__LIBEDB__)
#define	EdbTupleMakeCaptionFunc_P	__P((EdbContext, EdbTupleInfo, int, int, Link))
Caption		edb_tuple_make_caption_func		EdbTupleMakeCaptionFunc_P;
#endif /* defined(__LIBEDB__) */

Caption		edb_tuple_make_specified_caption	__P((EdbContext, EdbTupleInfo, int, Link));
int			edb_tuple_expand_check			__P((EdbContext, EdbTupleInfo));
EdbCatalogue	edb_tuple_expand				__P((EdbContext, EdbTupleInfo));
Link			edb_tuple_diff_columns			__P((EdbContext, EdbTupleInfo, EdbTupleInfo));

#define	EdbTuplePrintAdditionalInformation_P		__P((EdbContext, EdbTupleInfo, Doc))
int			edb_tuple_print_additional_information	EdbTuplePrintAdditionalInformation_P;

struct	EDB_TUPLE_CB_DATA	{
	void	*user_data;
};
typedef	struct EDB_TUPLE_CB_DATA	*EdbTupleCBData;

typedef	int	(*TCD_callback_func_t)			__P((EdbContext, EdbTupleInfo, int, EdbTupleColumn, EdbDatum, EdbTupleCBData));
void			edb_tuple_TCD_callback			__P((EdbContext, EdbTupleInfo, EdbTupleColumn, TCD_callback_func_t, void *));

typedef	int	(*TC_callback_func_t)			__P((EdbContext, EdbTupleInfo, int, EdbTupleColumn, EdbTupleCBData));
void			edb_tuple_TC_callback			__P((EdbContext, EdbTupleInfo, EdbTupleColumn, TC_callback_func_t, void *));

eid_t		edb_tuple_hierarchyparent				__P((EdbContext, EdbTupleInfo));
EdbDatum	edb_tuple_hierarchyparent_datum		__P((EdbContext, EdbTupleInfo));

int			edb_tuple_has_history			__P((EdbContext, EdbTupleInfo));

int	edb_tuple_has_conductor					__P((EdbContext, EdbTupleInfo));
int	edb_tuple_column_usage					__P((EdbContext, EdbTupleInfo, EdbColumnInfo));
int	edb_tuple_named_column_usage			__P((EdbContext, EdbTupleInfo, utf8_t *));

int	edb_tuple_get_minimum_date				__P((EdbContext, EdbTupleInfo));

#endif	/* Tuple_Inc_Defined */

