/*
$Header: /usr/staff/alex/edb/srcs/libedbpub/image/RCS/image.h,v 1.6 2004/03/30 13:05:53 alex Exp alex $
	image.h--- head file of image
	(C) Copyright 1989 by Alex
*/

#ifndef	Image_Inc_Defined
#define	Image_Inc_Defined

#ifdef	linux
#include	<stdlib.h>
#endif
#include	<sys/types.h>
#include	<sys/file.h>

#ifndef	TRUE
#define	TRUE	(1)
#endif

#ifndef	FALSE
#define	FALSE	(0)
#endif

#ifndef	ERROR
#define	ERROR	(-1)
#endif

#ifndef	min
#define	min(x, y)	((x)<(y)?(x):(y))
#endif

#ifndef	max
#define	max(x, y)	((x)>(y)?(x):(y))
#endif

#ifndef	sqr
#define	sqr(x)		((x)*(x))
#endif

struct image_info {
	int version;		/* version number */
	int flags;			/* reserve flags */

	/*
	pixel information
	*/
#define	IMAGE_TMASK	0x000000ff
#define	IMAGE_S		0x00000000	/* signed integer		*/
#define	IMAGE_U		0x00000001	/* unsigned integer		*/
#define	IMAGE_F		0x00000002	/* floating point		*/
#define	IMAGE_C		0x00000003	/* complex floating		*/
#define	IMAGE_VMASK	0x0000ff00
#define	IMAGE_VSHIFT	8
	int ptype;			/* mode of pixel data */
	int pbits;			/* pixel bits */
	int values;			/* nubmer of values in multi-value image */

	/*
	picture size information
	*/
	int xsize;			/* xsize (pixels) */
	int ysize;			/* ysize (lines) */
	int xbytes;			/* xbytes per one line */
};

struct image_header {
	struct image_info info;
	char dummy[256-sizeof(struct image_info)];
	char comment[256];
};

#define	IMAGE_VALUE	double

struct image_instance {
	struct image_header h;
	void	* body;
	IMAGE_VALUE	(*get_pixel)();
	void		(*put_pixel)();
	IMAGE_VALUE	minimum;
	IMAGE_VALUE	maximum;
};

/*#image_get_pixel--- get value of pixel from image */
/*#image_put_pixel--- put value of pixel to image */

#define	image_get_mpixel(i,x,y,n)	((*(i)->get_pixel)((i), (x), (y), (n)))
#define	image_put_mpixel(i,x,y,n,v)	((*(i)->put_pixel)((i), (x), (y), (n), (IMAGE_VALUE)(v)))

#define	image_get_pixel(i,x,y)		image_get_mpixel((i), (x), (y), 0)
#define	image_put_pixel(i,x,y,v)	image_put_mpixel((i), (x), (y), 0, (v))
#define	image_get_pixel1(i,x,y)		image_get_mpixel((i), (x), (y), 0)
#define	image_put_pixel1(i,x,y,v)	image_put_mpixel((i), (x), (y), 0, (v))
#define	image_get_pixel2(i,x,y)		image_get_mpixel((i), (x), (y), 1)
#define	image_put_pixel2(i,x,y,v)	image_put_mpixel((i), (x), (y), 1, (v))
#define	image_get_pixel3(i,x,y)		image_get_mpixel((i), (x), (y), 2)
#define	image_put_pixel3(i,x,y,v)	image_put_mpixel((i), (x), (y), 2, (v))

#define	image_minimum(i)			((i)->minimum)
#define	image_maximum(i)			((i)->maximum)

#define	IMAGE struct image_instance

typedef IMAGE *Image;

/**********/
/* macros */
/**********/
/*#image_version--- get image version */
#define	image_version(ip)	(((struct image_info *)ip)->version)

/*#image_ptype--- get pixel type of image */
#define	image_ptype(ip)		((((struct image_info *)ip)->ptype) & IMAGE_TMASK)

/*#image_make_ptype--- make pixel type */
#define	image_make_ptype(p, n, a, b)	((p)|((n)<<IMAGE_VSHIFT))

/*#image_pbits--- get number of bits of pixel */
#define	image_pbits(ip)		(((struct image_info *)ip)->pbits)

/*#image_is_mono--- Is the image monochrome ? */
#define	image_is_mono(ip)	(((struct image_info *)ip)->values <= 1)

/*#image_is_multi--- Is the image maluti-value */
#define	image_is_multi(ip)	(!image_is_mono(image))

/*#image_values--- get number of values of pixel */
#define	image_values(ip)	(((struct image_info *)ip)->values)

/*#image_xsize--- get xsize of image */
#define	image_xsize(ip)		(((struct image_info *)ip)->xsize)

/*#image_ysize--- get ysize of image */
#define	image_ysize(ip)		(((struct image_info *)ip)->ysize)

/*#image_xbytes--- get number of bytes of x-dir of raster scanned image */
#define	image_xbytes(ip)	(((struct image_info *)ip)->xbytes)

/*#image_bytes--- get number of bytes of image */
#define	image_bytes(ip)		(((struct image_info *)ip)->xbytes*((struct image_info *)ip)->ysize)

/*#image_pixels--- get number of pixels of image */
#define	image_pixels(ip)	(((struct image_info *)ip)->xsize*((struct image_info *)ip)->ysize)

/*#image_comment--- get comment of image */
#define	image_comment(ip)	(((struct image_header *)ip)->comment)

/*#image_get_comment--- set comment of image */
#define	image_get_comment(ip)	image_comment(ip)

/*#image_set_comment--- set comment of image */
int image_set_comment();

/*************/
/* functions */
/*************/
/*#image_dup--- duplicate image									*/
/*	new = image_dup(org)										*/
/*	Image new, org;												*/
/*	If new is NULL, duplication of image is failed.				*/
Image image_dup();

/*#image_load--- load image from file							*/
/*	image = image_load(name)									*/
/*	Image image;												*/
/*	char *name;													*/
/*	If image is NULL, loading is failed.						*/
Image image_load();

/*#image_load_header--- load only header of image from file		*/
/*	image = image_load(name)									*/
/*	Image image;												*/
/*	char *name;													*/
/*	If image is NULL, loading is failed.						*/
/*	<CAUTION> Body of image is not loaded.						*/
Image image_load_header();

/*#image_save--- save image to file								*/
/*	result = image_save(name, image)							*/
/*	Image result, image;										*/
/*	char *name;													*/
/*	If result is NULL, saving is failed.						*/
Image image_save();

/*#image_alloc--- allocate image								*/
/*	image = image_alloc(ptype, pbits, xsize, ysize, comment)	*/
/*	Image image;												*/
/*	int ptype;			...pixel type of image					*/
/*	int pbits;			...number of bits of pixel				*/
/*	int xsize, ysize;	... size of image						*/
/*	char *comment;		...comment								*/
/*	If image is NULL, image allocation is failed.				*/
Image image_alloc();

/*#image_free--- free image										*/
/*	result = image_free(image)									*/
/*	Image result, image;										*/
/*	Result is NULL.												*/
Image image_free();

Image image_conv();
Image image_conv_type();
Image image_transpose();
Image image_resize();
Image image_smooth();
Image image_median();
Image image_reverse();
Image raster_load();
Image image_to_mono();
Image image_get_region();
int   image_put_region();
Image image_rotate();
IMAGE_VALUE image_max_value();
IMAGE_VALUE image_min_value();
IMAGE_VALUE	image_sum();
IMAGE_VALUE	image_mean();
Image image_dith();

void	image_set_pixel_func	__P((Image image));
int	image_support			__P((Image image));

void	image_print_info	__P((Image image));

#endif	/* Image_Inc_Defined */

